;This procedure is intended for use as part of the CHIAS system.  This system
;has been developed by the HAO CHIP project to facilitate the initial
;quality assurance procedures required for new CHIP data.  Please make
;no changes to any of the files in this directory without first contacting
;CHIP personnel.
;
;   Questions regarding CHIAS should be directed to:
;
;			Ray Wrigley     FL2 3084  x1588
;			Alice Lecinski  FL2 3037  x1523
;
;   The purpose of this module is to sort through the directory where
;   the current day's data is stored and sort the filenames into arrays
;   of strings, each of  which contain all the filenames of one type of
;   image (e.g. hsr, hbr, etc.).   Also, this is a convenient time to
;   set up string arrays and such for the log file entry process.
;
pro chias_filesort
@chias_common

spawn, '(cd ' + maind + dataday + $
       '; /bin/ls -1 [09]?d???.??????.???)', files

files = files(1:n_elements(files) - 1)	;Strip first element- don't need it.

hsr_f_subs = where(strmid(files, 14, 3) eq 'hsr')
csr_f_subs = where(strmid(files, 14, 3) eq 'csr')

hsc_f_subs = where(strmid(files, 14, 3) eq 'hsc')
csc_f_subs = where(strmid(files, 14, 3) eq 'csc')
hsh_f_subs = where(strmid(files, 14, 3) eq 'hsh')
bsh_f_subs = where(strmid(files, 14, 3) eq 'bsh')

hfr_f_subs = where(strmid(files, 14, 3) eq 'hfr')
cfr_f_subs = where(strmid(files, 14, 3) eq 'cfr')

hbr_f_subs = where(strmid(files, 14, 3) eq 'hbr')
ssr_f_subs = where(strmid(files, 14, 3) eq 'ssr')
wsr_f_subs = where(strmid(files, 14, 3) eq 'wsr')

paint_f_subs = where(strmid(files, 15, 2) eq 'pt')

;If there are files of a certain type, then set up a string array of their
;names, and set up the logfile housekeeping stuff.


if (hsr_f_subs(0) ne -1) then begin
   hsr_fnms = files(hsr_f_subs)
   nm = n_elements (hsr_fnms)
   hsr_cmnts = strarr(nm) & hsr_done = intarr(nm) & hsr_qual = strarr(nm)
endif else begin
   hsr_done = [1] & hsr_fnms = 'n' 
   hsr_cmnts = ' '        & log_done(0) = 1       & hsr_qual = ' '
endelse

if csr_f_subs(0) ne -1 then begin
   csr_fnms = files(csr_f_subs)
   nm = n_elements (csr_fnms)
   csr_cmnts = strarr(nm) & csr_done = intarr(nm) & csr_qual = strarr(nm)
endif else begin
   csr_done = [1] & csr_fnms = 'n' 
   csr_cmnts = ' '        & log_done(1) = 1       & csr_qual = ' '
endelse

if hsc_f_subs(0) ne -1 then begin
   hsc_fnms = files(hsc_f_subs)
   nm = n_elements (hsc_fnms)
   hsc_cmnts = strarr(nm) & hsc_done = intarr(nm) & hsc_qual = strarr(nm)
endif else begin
   hsc_done = [1] & hsc_fnms = 'n' 
   hsc_cmnts = ' '        & log_done(3) = 1       & hsc_qual = ' '
endelse

if csc_f_subs(0) ne -1 then begin
   csc_fnms = files(csc_f_subs)
   nm = n_elements (csc_fnms)
   csc_cmnts = strarr(nm) & csc_done = intarr(nm) & csc_qual = strarr(nm)
endif else begin
   csc_done = [1] & csc_fnms = 'n' 
   csc_cmnts = ' '        & log_done(4) = 1       & csc_qual = ' '
endelse

if hsh_f_subs(0) ne -1 then begin
   hsh_fnms = files(hsh_f_subs)
   nm = n_elements (hsh_fnms)
   hsh_cmnts = strarr(nm) & hsh_done = intarr(nm) & hsh_qual = strarr(nm)
endif else begin
   hsh_done = [1] & hsh_fnms = 'n' 
   hsh_cmnts = ' '        & log_done(5) = 1       & hsh_qual = ' '
endelse

if bsh_f_subs(0) ne -1 then begin
   bsh_fnms = files(bsh_f_subs)
   nm = n_elements (bsh_fnms)
   bsh_cmnts = strarr(nm) & bsh_done = intarr(nm) & bsh_qual = strarr(nm)
endif else begin
   bsh_done = [1] & bsh_fnms = 'n' 
   bsh_cmnts = ' '        & log_done(16) = 1       & bsh_qual = ' '
endelse

if hfr_f_subs(0) ne -1 then begin
   hfr_fnms = files(hfr_f_subs)
   nm = n_elements (hfr_fnms)
   hfr_cmnts = strarr(nm) & hfr_done = intarr(nm) & hfr_qual = strarr(nm)
endif else begin
   hfr_done = [1] & hfr_fnms = 'n' 
   hfr_cmnts = ' '        & log_done(6) = 1       & hfr_qual = ' '
endelse

if cfr_f_subs(0) ne -1 then begin
   cfr_fnms = files(cfr_f_subs)
   nm = n_elements (cfr_fnms)
   cfr_cmnts = strarr(nm) & cfr_done = intarr(nm) & cfr_qual = strarr(nm)
endif else begin
   cfr_done = [1] & cfr_fnms = 'n' 
   cfr_cmnts = ' '        & log_done(7) = 1       & cfr_qual = ' '
endelse

if hbr_f_subs(0) ne -1 then begin
   hbr_fnms = files(hbr_f_subs)
   nm = n_elements (hbr_fnms)
   hbr_cmnts = strarr(nm) & hbr_done = intarr(nm) & hbr_qual = strarr(nm)
endif else begin
   hbr_done = [1] & hbr_fnms = 'n' 
   hbr_cmnts = ' '        & log_done(12) = 1       & hbr_qual = ' '
endelse

if ssr_f_subs(0) ne -1 then begin
   ssr_fnms = files(ssr_f_subs)
   nm = n_elements (ssr_fnms)
   ssr_cmnts = strarr(nm) & ssr_done = intarr(nm) & ssr_qual = strarr(nm)
endif else begin
   ssr_done = [1] & ssr_fnms = 'n' 
   ssr_cmnts = ' '        & log_done(13) = 1       & ssr_qual = ' '
endelse

if wsr_f_subs(0) ne -1 then begin
   wsr_fnms = files(wsr_f_subs)
   nm = n_elements (wsr_fnms)
   wsr_cmnts = strarr(nm) & wsr_done = intarr(nm) & wsr_qual = strarr(nm)
endif else begin
   wsr_done = [1] & wsr_fnms = 'n' 
   wsr_cmnts = ' '        & log_done(14) = 1       & wsr_qual = ' '
endelse

if paint_f_subs(0) ne -1 then begin
   paint_fnms = files(paint_f_subs)
   nm = n_elements (paint_fnms)
   paint_cmnts = strarr(nm) & paint_done = intarr(nm) & paint_qual = strarr(nm)
endif else begin
   paint_done = [1] & paint_fnms = 'n' 
   paint_cmnts = ' '        & log_done(15) = 1       & paint_qual = ' '
endelse






;Finally, and this sucks, there's really no better way to get the start and
;stop time for the log files than to get it from the filenames.
files_trim = files(where(strmid(files, 0, 1) eq '0'))
tm_str = strmid(files_trim, 7, 6)
hrs_str = fix(strmid(tm_str, 0, 2))
min_str = fix(strmid(tm_str, 2, 2))
rollover = where(hrs_str lt 10)		;A few lines to add 24 hrs to where
if rollover(0) ne -1 then begin		;the clock goes past midnight.
   hrs_str(rollover) = hrs_str(rollover + 24)
endif

tm_vals = float(hrs_str + min_str/60.)
max_ix = (where(tm_vals eq max(tm_vals)))(0)
min_ix = (where(tm_vals eq min(tm_vals)))(0)

stop_time = strmid(tm_str(max_ix), 0, 2)+':'+strmid(tm_str(max_ix), 2, 2)
start_time =  strmid(tm_str(min_ix), 0, 2)+':'+strmid(tm_str(min_ix), 2, 2)

sav_comm = ' '

return
end
