;This procedure is intended for use as part of the CHIAS system.  This system
;has been developed by the HAO CHIP project to facilitate the initial
;quality assurance procedures required for new CHIP data.  Please make
;no changes to any of the files in this directory without first contacting
;CHIP personnel.
;
;   Questions regarding CHIAS should be directed to:
;
;			Ray Wrigley     FL2 3084  x1588
;			Alice Lecinski  FL2 3037  x1523
;
;   The purpose of this module is to respond to the user's demand as to
;   which image to view next in the currently loaded image type.
;
pro chias_imchange, val
@chias_common

;Erase title screen if this is the first time through.
if title eq 1 then begin
   plot, [0, 0], [0, 0]
   chias_colorscale, 'RED'
   title = 0
endif

;Erase enlarged image if one was present in plot window.
if enlarged eq 1 then begin
   plot, [0, 0], [0, 0] 
   widget_control, col1eb, sensitive = 0
   enlarged = 0
endif

;Re-enable logfile controls if continuing after recalling image from
;another image type (see CHIAS_RECALL.PRO).
if wrongtyp eq 1 then begin
   wrongtyp = 0
   widget_control, col2b, sensitive = 1
   widget_control, usr_out1, set_value = $
   'Optionally comment <ret> and then select quality, limit 720 chars.'
endif

case val of 
   'init'     : ii = 0		;Initialize counter on first image of set.
   'PREV'     : begin
		    ii = ii - 1
		    if ii lt 0 then ii = n_imgs - 1
	        end
   'NEXT'     : ii = (ii +  1)mod(n_imgs)
   'FFWD'     : ii = (ii + 10)mod(n_imgs)
   'REWD'     : begin
		    ii = ii - 10
		    if ii lt 0 then begin
			repeat ii = n_imgs + ii until ii ge 0
		    endif
	        end
   'NXLOG'    : ii = next_intyp
   'REDRAW'   : ii = ii
   'OVERRIDE' : begin
		    diff = abs(slide_set - tmarr)
		    tmp_ii = where(diff eq min(diff))
		    ii = tmp_ii(0)
		end
   'ANIMATE'  : for w=0,n_imgs-1 do $
		    device,copy=[0,0,xsz,ysz,wxp,wyp,pwin_inds(w)]
endcase

;Copy from pixmap to realized draw widget.
device, copy = [0, 0, xsz, ysz, wxp, wyp, pwin_inds(ii)]

;Display any log entries already pending for chosen image.
case cur_typ of
   'hsr' : begin
	       widget_control, usr_out2, set_value = hsr_qual(ii)
	       widget_control, usr_in1,  set_value = hsr_cmnts(ii)
	    end
   'csr' : begin
	       widget_control, usr_out2, set_value = csr_qual(ii)
	       widget_control, usr_in1,  set_value = csr_cmnts(ii)
	    end
   'hsc' : begin
	       widget_control, usr_out2, set_value = hsc_qual(ii)
	       widget_control, usr_in1,  set_value = hsc_cmnts(ii)
	    end
   'csc' : begin
	       widget_control, usr_out2, set_value = csc_qual(ii)
	       widget_control, usr_in1,  set_value = csc_cmnts(ii)
	    end
   'hsh' : begin
	       widget_control, usr_out2, set_value = hsh_qual(ii)
	       widget_control, usr_in1,  set_value = hsh_cmnts(ii)
	    end
   'bsh' : begin
	       widget_control, usr_out2, set_value = bsh_qual(ii)
	       widget_control, usr_in1,  set_value = bsh_cmnts(ii)
	    end
   'hfr' : begin
	       widget_control, usr_out2, set_value = hfr_qual(ii)
	       widget_control, usr_in1,  set_value = hfr_cmnts(ii)
	    end
   'cfr' : begin
	       widget_control, usr_out2, set_value = cfr_qual(ii)
	       widget_control, usr_in1,  set_value = cfr_cmnts(ii)
	    end
   'hbr' : begin
	       widget_control, usr_out2, set_value = hbr_qual(ii)
	       widget_control, usr_in1,  set_value = hbr_cmnts(ii)
	    end
   'ssr' : begin
	       widget_control, usr_out2, set_value = ssr_qual(ii)
	       widget_control, usr_in1,  set_value = ssr_cmnts(ii)
	    end
   'wsr' : begin
	       widget_control, usr_out2, set_value = wsr_qual(ii)
	       widget_control, usr_in1,  set_value = wsr_cmnts(ii)
	    end
   'paint' : begin
	       widget_control, usr_out2, set_value = paint_qual(ii)
	       widget_control, usr_in1,  set_value = paint_cmnts(ii)
	    end
endcase

;Section to output to user and header information widgets.
widget_control, curim,   set_value = string(ii)
widget_control, imtyp,   set_value = cur_typ

widget_control, obsdat,  set_value = info_set(ii, 0)
widget_control, obstim,  set_value = info_set(ii, 1)
widget_control, nexpos,  set_value = info_set(ii, 2)
widget_control, hotpix,  set_value = string(hot_pix(ii))
widget_control, coldpix, set_value = string(cold_pix(ii))

;Set time slider to correct position.
nm   = n_imgs - 1
mint = float(strmid(fnms(0),  7, 2)) + float(strmid(fnms(0), 9,2))/60. + $
       float(strmid(fnms(0), 11,2))/(60.*60.)
maxt = float(strmid(fnms(nm), 7, 2)) + float(strmid(fnms(nm),9,2))/60. + $
       float(strmid(fnms(nm), 11,2))/(60.*60.)
nowt = float(strmid(fnms(ii), 7, 2)) + float(strmid(fnms(ii),9,2))/60. + $
       float(strmid(fnms(ii), 11,2))/(60.*60.)

if(maxt lt mint)then maxt=maxt+24.
if(nowt lt mint)then nowt=nowt+24.

tim_val = fix(1000 * (nowt-mint)/(maxt-mint))

widget_control, time_slide, set_value = tim_val

return
end
