;This procedure is intended for use as part of the CHIAS system.  This system
;has been developed by the HAO CHIP project to facilitate the initial
;quality assurance procedures required for new CHIP data.  Please make
;no changes to any of the files in this directory without first contacting
;CHIP personnel.
;
;   Questions regarding CHIAS should be directed to:
;
;			Ray Wrigley     FL2 3084  x1588
;			Alice Lecinski  FL2 3037  x1523
;
;   This module displays the next image of the type requested, loading
;   images of that type to RAM in reduced form - xsz X  ysz rebinned.
;
pro chias_load_images, val
@chias_common

datain = 0			;Flag for use later.

;Clear out windows if already in use.
if pwin_inds(0) ne 0 then for v = 0, n_imgs-1 do wdelete, pwin_inds(v)

;Clear out header info widgets during load.
widget_control, obsdat,  set_value =  ' '
widget_control, obstim,  set_value = ' '
widget_control, nexpos,  set_value = ' '
widget_control, hotpix,  set_value = ' '
widget_control, coldpix, set_value = ' '

if(not n_elements(im_array)) then  im_array=intarr(xsz,ysz)

case val of			;Get number of images of selected type.
   'hsr'  : begin & n_imgs = n_elements(hsr_fnms) & fnms = hsr_fnms & end
   'csr'  : begin & n_imgs = n_elements(csr_fnms) & fnms = csr_fnms & end

   'hsc'  : begin & n_imgs = n_elements(hsc_fnms) & fnms = hsc_fnms & end
   'csc'  : begin & n_imgs = n_elements(csc_fnms) & fnms = csc_fnms & end
   'hsh'  : begin & n_imgs = n_elements(hsh_fnms) & fnms = hsh_fnms & end
   'bsh'  : begin & n_imgs = n_elements(bsh_fnms) & fnms = bsh_fnms & end

   'hfr'  : begin & n_imgs = n_elements(hfr_fnms) & fnms = hfr_fnms & end
   'cfr'  : begin & n_imgs = n_elements(cfr_fnms) & fnms = cfr_fnms & end

   'hbr'  : begin & n_imgs = n_elements(hbr_fnms) & fnms = hbr_fnms & end
   'ssr'  : begin & n_imgs = n_elements(ssr_fnms) & fnms = ssr_fnms & end
   'wsr'  : begin & n_imgs = n_elements(wsr_fnms) & fnms = wsr_fnms & end

   'paint'  : begin & n_imgs = n_elements(paint_fnms) & fnms = paint_fnms & end
endcase

if strtrim(fnms(0), 2) eq 'n' then goto, NOFILE

hdrs     = ' '		        ;Nucleus for fits header storage.
hot_pix  = intarr(n_imgs)	;Array to save maximum pixels in images.
cold_pix = intarr(n_imgs)	;Array to save minimum pixels in images.

widget_control, totim, set_value = string(n_imgs)	;Inform user.

pwin_inds = intarr(n_imgs)	;Vector to hold pixwin indexes.

get_lun, lun

for t = 0, n_imgs - 1 do begin
   widget_control, usr_out1, set_value = 'Loading image ' + string(t)
   im_tmp = readimg (maind + dataday + '/' + fnms(t), pixx,pixy)

  ;im_tmp = readfits('/puka/f/cordyn/chippro/' + dataday + '/' + fnms(t), fitshdr)
   ;hdrs = [hdrs, fitshdr]	;Add to string holding all header info.
   ;hot_pix(t)  = max(im_tmp(400:pixx-400,200:pixy-200)) ;Save max and min.
   ;cold_pix(t) = min(im_tmp(400:pixx-400,200:pixy-200))

   hot_pix(t)  = max(im_tmp(200:pixx-200,200:pixy-200)) ;Save max and min.
   cold_pix(t) = min(im_tmp(200:pixx-200,200:pixy-200))

   std_hot  = hot_pix(t)
   std_cold = cold_pix(t)

   if( val eq 'paint' and datascale eq 1) then begin
       std_hot  = 8192+100
       std_cold = 8192-100
   endif
   if( val eq 'hbr' and datascale eq 1) then begin
       std_hot  = 0+20
       std_cold = 0-20
   endif
   if( val eq 'hsh' ) then begin
       if( datascale eq 1) then begin
	   ;std_hot  =  100
	   std_hot  =  750  ;added by Tony, use for 7 filter position images.
	   std_hot  =  200
	   ;std_cold = -600 
	   std_cold = -2000 ;added by Tony, use for 7 filter position images.
	   std_cold = -600 
       endif	else begin
	   outside = float(total(im_tmp(50:300,50:300)))/$
				((300.-49.)*(300.-49.))
	   inside  = float(total(im_tmp(400:pixx-400,200:pixy-200)))/ $
				float((pixx-400-400)*(pixy-200-200))

	   outside = min(im_tmp( 50:300     , 50:300     ))
	   inside  = min(im_tmp(400:pixx-400,200:pixy-200))

	   hot_pix (t) = outside
	   cold_pix(t) = inside
       endelse
   endif
   if( val eq 'bsh' ) then begin
       if( datascale eq 1) then begin
	   ;std_hot  =  200
	   std_hot  =  850  ;added by Tony, use for 7 filter position images.
	   std_hot  =  200  ;added by Tony, use for 7 filter position images.
	   ;std_cold = -200
	   std_cold = -650 ;added by Tony, use for 7 filter position images.
       endif	else begin
	   outside = float(total(im_tmp(50:300,50:300)))/$
				((300.-49.)*(300.-49.))
	   inside  = float(total(im_tmp(400:pixx-400,200:pixy-200)))/ $
				float((pixx-400-400)*(pixy-200-200))

	   outside = min(im_tmp( 50:300     , 50:300     ))
	   inside  = min(im_tmp(400:pixx-400,200:pixy-200))

	   hot_pix (t) = outside
	   cold_pix(t) = inside
       endelse
   endif

;	In the rebinning below, the top row and some right columns
;	of im_tmp is dropped in order to make the array dimensions even
;	multiples of three.  This is necessary for the rebin routine,
;	which requires the result array to have dimensions which are
;	factors or multiples of the original array.

   im_array = rebin(im_tmp(0:xsz*scl-1,0:ysz*scl-1),xsz,ysz)

   window, /free, xsize = xsz, ysize = ysz, /pixmap	;Create pixmap.
   pwin_inds(t) = !d.window			;Get pixmap index.
   wset, pwin_inds(t)				;Direct output to pixmap.

   case datascale of		;Write image to pixmap according to scaling.
      0 :     tv, bytscl(im_array, min=cold_pix(t), max=hot_pix(t))
      1 :     tv, bytscl(im_array, min=std_cold   , max=std_hot   )
   endcase

   wset, plot_index1	;Set output to draw widget to quickly display image.
   device, copy = [0, 0, xsz , ysz , wxp , wyp , pwin_inds(t)]
   widget_control, curim, set_value = string(t)
endfor

datain = 1		;Flag to indicate data was loaded, used below.

free_lun, lun

;Set up time arrays for eventual reading of the time slider.
hrarr  = float(strmid(fnms, 7, 2))
minarr = float(strmid(fnms, 9, 2))
tmarr  = (hrarr + minarr/60.) - min(hrarr + minarr/60.)
tmarr  = tmarr * (1000./max(tmarr))

wset, plot_index1		;Set output to draw widget.

NOFILE:
if datain ne 1 then begin
   widget_control, totim, set_value = ' '
   widget_control, usr_out1, set_value = 'No files of that type today.'
endif

return
end
