;This procedure is intended for use as part of the CHIAS system.  This system
;has been developed by the HAO CHIP project to facilitate the initial
;quality assurance procedures required for new CHIP data.  Please make
;no changes to any of the files in this directory without first contacting
;CHIP personnel.
;
;   Questions regarding CHIAS should be directed to:
;
;                        Ray Wrigley     FL2 3084  x1588
;                        Alice Lecinski  FL2 3037  x1523
;
;   This file contains two procedures.  The first, chias_wgt_event, is the
;   event handler for the widgets interface on which this system is based.
;   The second is chias_wgt_create, which specifies the appearance of the
;   control panel through which the user operates the CHIAS system.  
;   Changes made in one procedure must be reflected in the other for the
;   system to work as designed.
;
pro chias_wgt_event, ev
@chias_common
on_error, 0

widget_control, ev.id, get_uvalue = uval        ;Extract widget user value.

;Don't get value from value-unreadable widgets.  Check before extracting val.
val_test = strmid(uval, 3, 1)
if val_test ne 'a' and val_test ne 'b' and val_test ne 'c'$
   then widget_control, ev.id, get_value = val

;Widgets are identified specifically with uval.  The following large case
;statement directs input from the current widget event  according to this
;value.

if(uval ne 'op99' and uval ne 'op99b' )then begin ;{
    if(realexit eq 1) then widget_control, usr_out1, set_value =' '
    if(uval ne 'op99a' )then widget_control, col1fb, sensitive = 0
    realexit=0
endif;}

case uval of
   'op1'  : begin
              chias_colorscale, 'RED'        ;Load default colorscale.
              plot, [0, 0], [0, 0]                ;Erase title screen.
              if val eq 'MIN-MAX' then datascale = 0 else datascale = 1
              widget_control, col1b,  sensitive = 1
              widget_control, col1c,  sensitive = 0
              widget_control, col1d,  sensitive = 0
              widget_control, col1e,  sensitive = 0
              widget_control, col2b,  sensitive = 1
              widget_control, col2ba, sensitive = 1
              widget_control,usr_out1,set_value=$
                  'Select image type to view, or enter summaries.'
            end
   'op2'  : begin
              cur_typ = val
              widget_control, imtyp, set_value = val
              widget_control, curim, set_value = ' '
              widget_control, usr_out2, set_value = ' '
              widget_control, nxlog, set_value = '      0'
              chias_load_images, val
              if strtrim(fnms(0), 2) ne 'n' then begin
                 chias_parse_hdrs
                 chias_imchange, 'init'
                 widget_control, col1c,   sensitive = 1 
                 widget_control, col1d,   sensitive = 1
                 widget_control, col1e,   sensitive = 1
                 widget_control, col2baa, sensitive = 1
                 widget_control, col2bad, sensitive = 1
                 widget_control, col2bae, sensitive = 1
                 widget_control, col2baf, sensitive = 1
                 widget_control, col2bag, sensitive = 1
                 widget_control, usr_out1, set_value = $
                 'Optionally comment <ret> , '+$
                 'and then select quality for each image, limit 720 characters.'
              endif
            end 
   'op3'  : chias_imchange,  val
   'op3a' : chias_imchange, 'REWD'
   'op3b' : chias_imchange, 'FFWD'
   'op3c' : chias_setgrid
   'op3s' : begin
               slide_set = val
               chias_imchange, 'OVERRIDE'
            end
   'op4'  : chias_colorscale, val
   'op5'  : begin
              widget_control, col1eb, sensitive = 1
              chias_enlarge, 1
            end
   'op5a' : chias_enlarge, 2
   'op5b' : chias_enlarge, 3
   'op5c' : chias_enlarge, 4
   'op6'  : chias_save, val
   'op7'  : chias_recall, val
   'op10' : begin
               case val of
                  '  G  ' : cur_qual = ' Good    '
                  '  B  ' : cur_qual = ' Bad     '   
                  '  U  ' : cur_qual = ' Ugly    '
               endcase
               widget_control, usr_out2, set_value = cur_qual
               comm_in = sav_comm
               print,'Comment in  ',comm_in
               chias_logenter, comm_in
               comm_in = ''
               sav_comm = ''
            end
   'op11' : begin
                comm_in  = (strmid(val, 0, cstrlen))(0)
                sav_comm = comm_in
                cur_summ = comm_in
                comm_in = ''
            end
   'op12' : begin
               case val of
                  'DAILY SUMMARY' : begin
                                 g_ix = where(strtrim(cur_summ,2) ne '')
                                 if g_ix(0) ne -1 then begin
                                    daily_summ = transpose(cur_summ(g_ix))
                                 endif else begin
                                    daily_summ = strarr(1)
                                    daily_summ(0) = 'No entry.'
                                 endelse
                                 das_done = 1
                                 widget_control, usr_out1, set_value =$
                                 'Daily summary entered.'
                                 wait, 1
                              end
                  'HELIUM EVENTS' : begin
                                 g_ix = where(strtrim(cur_summ,2) ne '')
                                 if g_ix(0) ne -1 then begin
                                    helium_events = transpose(cur_summ(g_ix))
                                 endif else begin
                                    helium_events = strarr(1)
                                    helium_events(0) = 'No entry.'
                                 endelse
                                 h_done = 1
                                 widget_control, usr_out1, set_value =$
                                 'Helium events summary entered.'
                                 wait, 1
                              end
               endcase
               comm_in  = ''
               sav_comm = comm_in
               cur_summ = comm_in
               widget_control, usr_in1, set_value = comm_in
               if(das_done eq 1 and h_done eq 1)then begin ;{
                   summs_done=1
                   widget_control, col2bac, sensitive = 0
                   widget_control, usr_out1, set_value =$
                       'Animate or enter another image type.'
               endif;}
            end
   'op13' : begin
               case cur_typ of
                  'hsh' : begin
                              hsh_arch_fnm = hsh_fnms(ii)
                              hsh_qual(ii) = ' ARCHIVE '
                              log_done(5) = 1
                              arch_in = 1
                              widget_control, usr_in1, set_value = ''
                           end
               endcase
               chias_logenter, ' '
            end
   'op14' : begin
               widget_control, base, sensitive = 0
               chias_logstatus
               widget_control, base, sensitive = 1
               chias_imchange, 'REDRAW'
            end
   'op99' : begin
               widget_control, col1fb, sensitive = 1
            end
   'op99a': begin
                widget_control, usr_out1, set_value =$
                      'Cleaning up -- might take a while --then exitting...'
                chias_writelogs
                widget_control, base, /destroy
            end
   'op99b': begin
                if(realexit eq 1)then begin
                    widget_control, base, /destroy
                endif else begin
                    realexit=1
                    widget_control, usr_out1, set_value =$
                    'If you really want to exit without saving the logs,'+$
                    ' hit clear again.'
                endelse
            end
endcase
end                                ;End of widget event handler.

;-----------------------------------------------------------------------------
;                        Begin widget creation routine.
;-----------------------------------------------------------------------------
pro chias_wgt_create
@chias_common

font  = '*times*bold*normal*14*'
font  = '*times*bold*normal*8*'
font  = '*times*bold*normal*12*'
afont = '*courier*18*'
afont2= '*courier*14*'
widget_control, default_font = font

;Create top-level base.
base  = widget_base(title = 'CHIP Image Assessment Software',$
                   /row, event_pro = 'chias_wgt_event')

col1  = widget_base(base, /column)
col2  = widget_base(base, /column)
col3  = widget_base(base, /column, xsize = 150)

;Begin column 1 layout.
;Define left side panels.
col1a = widget_base(col1, /column, frame = 2, ypad = 1, xpad = 10)        
col1b = widget_base(col1, /column, frame = 2, ypad = 1)
col1c = widget_base(col1, /column, frame = 2)
col1d = widget_base(col1, /column, frame = 2, ypad = 1, xpad = 10)
col1e = widget_base(col1, /column, frame = 2)
col1f = widget_base(col1, /column, frame = 2)

;Begin placing items in base col1a.
dm    = widget_label(col1a, value = 'DATA SCALING')
xmenu, ['MIN-MAX', 'STANDARD'], col1a,  uvalue = ['op1', 'op1'], /exclusive


;Begin placing items in base col1b.
dm    = widget_label(col1b, value = 'IMAGE TYPE')
paddy = 18
xmenu, ['hsr', 'hsc'],  col1b, uvalue=['op2','op2'],font=afont2,/row,xpad=paddy
xmenu, ['csr', 'csc'],  col1b, uvalue=['op2','op2'],font=afont2,/row,xpad=paddy
xmenu, ['hbr', 'hsh'],  col1b, uvalue=['op2','op2'],font=afont2,/row,xpad=paddy
xmenu, ['hfr', 'cfr'],  col1b, uvalue=['op2','op2'],font=afont2,/row,xpad=paddy
xmenu, ['ssr', 'wsr'],  col1b, uvalue=['op2','op2'],font=afont2,/row,xpad=paddy
xmenu, ['paint','bsh'], col1b, uvalue=['op2','op2'],font=afont2,/row,xpad=paddy


;Begin placing items in base col1c.
dm = widget_label(col1c, value = 'IMAGE SELECTION')
read_x11_bitmap, 'ffwd', ffwd        ;Read in fast-fwd and rewind bitmaps.
read_x11_bitmap, 'rewd', rewd

xmenu, ['PREV', 'NEXT'], col1c, /row, xpad = 18, uvalue = ['op3', 'op3']
col1ca = widget_base  (col1c, /row, xpad = 32)  ;Base for ffwd and rewd buttons.
rew    = widget_button(col1ca, value = rewd, uvalue = 'op3a')
fwd    = widget_button(col1ca, value = ffwd, uvalue = 'op3b')

time_slide = widget_slider(col1c, maximum=1000, uvalue='op3s', /suppress_value)
animate    = widget_button(col1c, value = 'ANIMATE'   , uvalue = 'op3')
grid       = widget_button(col1c, value = 'PLACE GRID', uvalue = 'op3c')


;Begin placing buttons in base col1d.
dm     = widget_label(col1d, value = 'IMAGE SCALING')
col1da = widget_base(col1d, /column, xpad = 10, /exclusive)
linbw  = widget_button(col1da, value = 'B/W LIN',   uvalue = 'op4')
exp1bw = widget_button(col1da, value = 'B/W EXP2',  uvalue = 'op4')
exp2bw = widget_button(col1da, value = 'B/W EXP5',  uvalue = 'op4')
log1bw = widget_button(col1da, value = 'B/W LG-5',  uvalue = 'op4')
log2bw = widget_button(col1da, value = 'B/W LG-20', uvalue = 'op4')
colr1  = widget_button(col1da, value = 'RED',       uvalue = 'op4')

widget_control, colr1, set_button = 1                ;Initially, set colr1


;Begin placing items in base col1e.
enlarge = widget_button(col1e, value = 'ENLARGE', uvalue = 'op5')
dm      = widget_label(col1e, value = 'IMAGE MOTION')
read_x11_bitmap, 'l_arrow', l_arrow        ;Read in arrow bitmaps.
read_x11_bitmap, 'r_arrow', r_arrow
read_x11_bitmap, 'im_cent', im_cent

;Create buttons with arrows in them.
col1eb = widget_base(col1e, /row, xpad = 20)
lfarr =  widget_button(col1eb, value = l_arrow, uvalue = 'op5a', xsize = 20)
imcen =  widget_button(col1eb, value = im_cent, uvalue = 'op5b', xsize = 20)
rtarr =  widget_button(col1eb, value = r_arrow, uvalue = 'op5c', xsize = 20)


dm     = widget_label (col1f, value = 'EXIT PROGRAM')
ext    = widget_button(col1f, value = 'EXIT', uvalue = 'op99')
col1fb = widget_base  (col1f, /row, xpad = 0)
appnd  = widget_button(col1fb, value = 'APPND', uvalue = 'op99a', xsize = 60)
clrlg  = widget_button(col1fb, value = 'CLEAR', uvalue = 'op99b', xsize = 60)


;Begin column 2 layout.
col2a     = widget_base(col2, /column)             ;Base for plotting window.
col2b     = widget_base(col2, /column, frame = 5)  ;Base for logfile controls.

;main_plot = widget_draw(col2a, xsize = wxsz, ysize = wysz, retain = 0)
main_plot = widget_draw(col2a, xsize = wxsz, ysize = wysz, retain = 2, $
	x_scroll_size=800, y_scroll_size=800,/scroll)

usr_mes1 = 'Please select the data scaling for the desired image type.'
usr_out1 = widget_text(col2b, val=usr_mes1, font=afont2)

col2ba   = widget_base(col2b, /row)

col2baa  = widget_base(col2ba, /column, frame = 5) ;Base for g/b/u selection.
dm       = widget_label(col2baa, value = 'IMAGE QUALITY')
xmenu,['  G  ','  B  ','  U  '], col2baa, uvalue = replicate('op10', 3),$
        /row, xpad = 8

usr_out2 = widget_text(col2ba, xsize = 5, ysize = 1, val = ' ')


col2bab = widget_base(col2ba, /column, frame = 5, ypad = 3)  ;Archive select.
arch_select = widget_button(col2bab, value = 'ARCHIVE', uvalue = 'op13')

col2bac = widget_base(col2ba, /column, frame = 5) 
xmenu, ['DAILY SUMMARY', 'HELIUM EVENTS'], col2bac,$
        uvalue = replicate('op12', 2), /column

;Row of three bases for save and recall buttons.
col2bad = widget_base(col2ba, /column, ypad = 5)
save1   = widget_button(col2bad, value = 'SAVE 1', uvalue = 'op6')
recall1 = widget_button(col2bad, value = 'RECALL 1', uvalue = 'op7')
col2bae = widget_base(col2ba, /column, ypad = 5)
save2   = widget_button(col2bae, value = 'SAVE 2', uvalue = 'op6')
recall2 = widget_button(col2bae, value = 'RECALL 2', uvalue = 'op7')
col2baf = widget_base(col2ba, /column, ypad = 5)
save3   = widget_button(col2baf, value = 'SAVE 3', uvalue = 'op6')
recall3 = widget_button(col2baf, value = 'RECALL 3', uvalue = 'op7')

col2bag = widget_base(col2ba, /column, frame = 5, ypad=3)
status  = widget_button(col2bag, value = 'LOG STATUS', uvalue = 'op14')

col2bb  = widget_base(col2b, /row, xpad = 50)
dm      = widget_label(col2bb, value = 'TEXT INPUT')
usr_in1 = widget_text(col2bb, value = ' ', xsize = 80, uvalue = 'op11',$
                      /editable)


;Begin column 3 layout.
col3a = widget_base(col3, /column, frame = 5) ;Base for user info/stats.
col3b = widget_base(col3, /column, frame = 5) ;Base for image info output.


;Begin placing items in base col3a.
dm =     widget_label(col3a, value = 'USER INFO'  , /frame)
dm =     widget_label(col3a, value = 'CURRENT DAY')
curday = widget_text (col3a, value = dataday      ,font=afont2)
dm =     widget_label(col3a, value = 'IMAGE TYPE' )
imtyp =  widget_text (col3a, value = ' '          ,font=afont2)
dm =     widget_label(col3a, value = 'TOTAL IMS'  )
totim =  widget_text (col3a, value = ' '          ,font=afont2)
dm =     widget_label(col3a, value = 'CURRENT IM' )
curim =  widget_text (col3a, value = ' '          ,font=afont2)
dm =     widget_label(col3a, value = 'NX FOR LOG' )
nxlog =  widget_text (col3a, value = '   0'       ,font=afont2)


;Begin placing items in base col3b.
dm      = widget_label(col3b, value = 'HEADER INFO', /frame)
dm      = widget_label(col3b, value = 'OBS DATE')
obsdat  = widget_text (col3b, value = ' ',                   font=afont2)
dm      = widget_label(col3b, value = 'OBS TIME')
obstim  = widget_text (col3b, value = ' ',                   font=afont2)
dm      = widget_label(col3b, value = '# EXPOSURES')
nexpos  = widget_text (col3b, value = ' ',                   font=afont2)
dm      = widget_label(col3b, value = 'MAX PIXEL')
hotpix  = widget_text (col3b, value = ' ',                   font=afont2)
dm      = widget_label(col3b, value = 'MIN PIXEL')
coldpix = widget_text (col3b, value = ' ',                   font=afont2)

widget_control, /realize, base                ;Creatus widgetus.
widget_control, get_value = plot_index1, main_plot        ;Get plot destination.

widget_control, col1b,   sensitive = 0
widget_control, col1c,   sensitive = 0
widget_control, col1d,   sensitive = 0
widget_control, col1e,   sensitive = 0
widget_control, col1eb,  sensitive = 0
widget_control, col1fb,  sensitive = 0
widget_control, col2b,   sensitive = 0
widget_control, col2ba , sensitive = 0
widget_control, col2baa, sensitive = 0
widget_control, col2bab, sensitive = 0
widget_control, col2bad, sensitive = 0
widget_control, col2bae, sensitive = 0
widget_control, col2baf, sensitive = 0
widget_control, col2bag, sensitive = 0

chias_title                ;Call routine to put up pretty opening screen.

xmanager, 'chias_wgt', base                ;Hand control to manager.

return
end
