;This procedure is intended for use as part of the CHIAS system.  This system
;has been developed by the HAO CHIP project to facilitate the initial
;quality assurance procedures required for new CHIP data.  Please make
;no changes to any of the files in this directory without first contacting
;CHIP personnel.
;
;   Questions regarding CHIAS should be directed to:
;
;			Ray Wrigley     FL2 3084  x1588
;			Alice Lecinski  FL2 3037  x1523
;
;   When all the information for the logfile is in, control is handed to
;   this routine, which then organizes the various string arrays and 
;   writes the actual log file, storing it the predetermined directory.
;   Also, this routine writes the archived helium data files to
;   the appropriate storage location.
;
pro chias_writelogs
@chias_common

;Set up arrays to hold lines of log file related to individual images.
hsr_strs = strarr(n_elements(hsr_fnms))
csr_strs = strarr(n_elements(csr_fnms))

hsc_strs = strarr(n_elements(hsc_fnms))
csc_strs = strarr(n_elements(csc_fnms))
hsh_strs = strarr(n_elements(hsh_fnms))
bsh_strs = strarr(n_elements(bsh_fnms))

hfr_strs = strarr(n_elements(hfr_fnms))
cfr_strs = strarr(n_elements(cfr_fnms))

hbr_strs = strarr(n_elements(hbr_fnms))
ssr_strs = strarr(n_elements(ssr_fnms))
wsr_strs = strarr(n_elements(wsr_fnms))

paint_strs = strarr(n_elements(paint_fnms))

;Create filename/comment lines of logfile.
for a = 0, n_elements(hsr_fnms) - 1 do begin
   hsr_strs(a) = hsr_fnms(a) + hsr_qual(a) + hsr_cmnts(a)
endfor

for a = 0, n_elements(csr_fnms) - 1 do begin
   csr_strs(a) = csr_fnms(a) + csr_qual(a) + csr_cmnts(a)
endfor

for a = 0, n_elements(hsc_fnms) - 1 do begin
   hsc_strs(a) = hsc_fnms(a) + hsc_qual(a) + hsc_cmnts(a)
endfor

for a = 0, n_elements(csc_fnms) - 1 do begin
   csc_strs(a) = csc_fnms(a) + csc_qual(a) + csc_cmnts(a)
endfor

for a = 0, n_elements(hsh_fnms) - 1 do begin
   hsh_strs(a) = hsh_fnms(a) + hsh_qual(a) + hsh_cmnts(a)
endfor

for a = 0, n_elements(bsh_fnms) - 1 do begin
   bsh_strs(a) = bsh_fnms(a) + bsh_qual(a) + bsh_cmnts(a)
endfor

for a = 0, n_elements(hfr_fnms) - 1 do begin
   hfr_strs(a) = hfr_fnms(a) + hfr_qual(a) + hfr_cmnts(a)
endfor

for a = 0, n_elements(cfr_fnms) - 1 do begin
   cfr_strs(a) = cfr_fnms(a) + cfr_qual(a) + cfr_cmnts(a)
endfor

for a = 0, n_elements(hbr_fnms) - 1 do begin
   hbr_strs(a) = hbr_fnms(a) + hbr_qual(a) + hbr_cmnts(a)
endfor

for a = 0, n_elements(ssr_fnms) - 1 do begin
   ssr_strs(a) = ssr_fnms(a) + ssr_qual(a) + ssr_cmnts(a)
endfor

for a = 0, n_elements(wsr_fnms) - 1 do begin
   wsr_strs(a) = wsr_fnms(a) + wsr_qual(a) + wsr_cmnts(a)
endfor

for a = 0, n_elements(paint_fnms) - 1 do begin
   paint_strs(a) = paint_fnms(a) + paint_qual(a) + paint_cmnts(a)
endfor

fnm_comm_array = [hsr_strs, ' ', csr_strs  , ' ', hsc_strs, ' ',$
                  csc_strs, ' ', hsh_strs  , ' ', hfr_strs, ' ',$
		  cfr_strs, ' ', hbr_strs  , ' ', ssr_strs, ' ',$
		  wsr_strs, ' ', paint_strs, ' ', bsh_strs, ' ' ]

;Yank out any n's which are artifacts of the absence of a file type.
if (where(strtrim(fnm_comm_array, 2) eq 'n'))(0) ne -1 then begin
   fnm_comm_array = fnm_comm_array(where(strtrim(fnm_comm_array, 2) ne 'n'))
endif

;Get the year of the image file and create the string for the path to save to.
endyr = strmid(dataday, 0, 2)
if fix(endyr) gt 78 then yr = '19' + endyr else yr = '20' + endyr
;path = '/jabba/ftp/chip/log/' + yr
path = '/toshi/ftp/d5/chip/log/' + yr

;Begin writing to output file.
get_lun, lun

tpfile = path + '/tmp.log'
lgfile = path + '/' + dataday + '.log'

openw, lun, tpfile

printf, lun, transpose(fnm_comm_array)
printf, lun, ' '
printf, lun, ' '
printf, lun, ' '
printf, lun, 'Start time: ' + start_time
printf, lun, 'Stop time : ' + stop_time
printf, lun, ' '
printf, lun, 'Daily Summary : '
printf, lun, '     '+daily_summ
printf, lun, ' '
printf, lun, 'Events : '
printf, lun, '     '+helium_events
printf, lun, ' '
cur_time_str = strmid(!stime, 0, 12)
printf, lun, 'These notes recorded by ' + op_name + ', ' + cur_time_str
close, lun

;cmnd='cat '+tpfile+' >> '+lgfile
cmnd='cat '+tpfile+' > '+lgfile
spawn,cmnd
cmnd='/bin/rm '+tpfile
spawn,cmnd

;Make second copy of logfile in /puu/f/cordyn/DATA_BASE/chiasLogs
cp_str = 'cp '+path+'/'+dataday+'.log'+' /puu/f/cordyn/DATA_BASE/chiasLogs/'+$
	dataday+'.log'
spawn, cp_str

;Make the daily MOVIE
cmnd = 'rsh wikiwiki /home/cordyn/mrchip/bin/doMovie_chias '+ dataday +' &'
spawn,cmnd

;Generate month name from day of year, for use below.
year = fix(yr)
day = fix(strmid(dataday, 3, 3))

mons=['01_jan','02_feb','03_mar','04_apr','05_may','06_jun','07_jul','08_aug','09_sep','10_oct','11_nov','12_dec']
mon_dys = [[0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334],$
	   [0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335]]

leap = ((year mod 4 eq 0) and (year mod 100 ne 0)) or (year mod 400 eq 0)

ix = where(day gt mon_dys(*, leap))
month = mons(max(ix))

;Copy archive image file to destination directory.

cp_cmd_hsh = 'cp ' + maind  + dataday + '/' + hsh_arch_fnm  +$
	     ;' /jabba/ftp/chip/daily_images/' + yr + '/' + month +'/' +$
	     ' /toshi/ftp/d5/chip/daily_images/' + yr + '/' + month +'/' +$
	     hsh_arch_fnm

print, cp_cmd_hsh
spawn, cp_cmd_hsh

return
end
