/*
 *  LDmodel.h
 *
 *  Header structure for CHIP limb dark modeling parameters.
 *  This header is contained in the second row of the calibrated "hs?" images.
 *  It contains the results of the fit for the limb darkening coefficients
 *     for both the helium and continuum images.   
 *  The standard limb dark equation is:

      (Brightness at R)/(BrightnessDiskCenter)
                                = a + b*cos(R) + c*(1.-cos(R)*ln(1.+1./cos(R)))
    or
      B/BC = a + b*cos(R) + c*(1.-cos(R)*ln(1.+1./cos(R)))

	    where R = arcsin( radialpositionpixels / Rsunpixels )

	    and   c = (1. -a -b ) / (0.306853 )
    
 *  The a  b   and   c   parameters are computed via least squares.
 *  See the code solve_LD_curve.c for more information.
 *  
 *  Alice Lecinski
 *
 *  Thu Aug 14 16:14:54 MDT 1997
 */


struct  LDparams {
    float pop_frac;   /* Population fraction at which to take the annuli
			 brightness measurement.                             */

    float a, b, c;    /* "a" "b" and "c" parameters for the limb dark model. */
                      /* "c"    is the brightness at the center of the disk. */

    float Index;      /* Sum of all (B/BC) / Nsunpix                         */
};

struct  LDmodel_info {

  float  Nannuli;      /* Number of annuli used to compute the model. */

  struct LDparams  Cp;  /* Continuum */
  struct LDparams  Hp;  /* Helium */


};


#ifdef START
  struct  LDmodel_info LDmodel;
#endif
