/*
    
    getchipflat goes through the "macpaint" process to find a 
    "flat" or "paint" chip image.
    The routine uses most of David Elmore's "filter" codes,
    but as subroutines instead of filters.

    Note: 
	It turns out that the positions of the centers varies less than
	the error produced by the code.  So, the centers and the dsk images
	need only be done once.  This is the (dofast) parameter...

	Fast vs Slow  were checked on 95d265 data.  Results are not identical
	because of the slightly shifting centers.  But to the eye, they look
	the same.  (37 minutes fast method, 43 minutes slow method, for 9
	hfr images -- The close numbers arrise because the fast method
	has to read and write stuff to disk, and the slow method
	runs computations, but holds everything in memory.)

    Basic methodology to produce the flat or "paint" image  is:

       I. Remove the bias.    *.b*  images.

       Create a fake "paint" image, which is 8192 everywhere (8192=NORM)

          1) Take each say *hfr* image and divide by *paint*
	     This flattens the images.  **.b.f**
          2) Find xyshifts of the true center from the expected center of the
	     image.  Find the solar diameter in pixels along the way.
	     (Routine "findshift")
          3) Center each flattened image with the xyshifts found above.
	     (Routine "do_shift")
	     These are "flattened-centered" images, or 
								   *.b.fc*
	     Write these out to disk, since we don't have enough room
	     in memory.
          4) Average all the **fc** images together, and normalize to
	     NORM along the way.  The routine is "norm_accumulate",
	     and it creates:                                       *solar*
	     *solar* should be predominated by solar features.

          5) Divide each *fc* by *solar*   
             This step removes common solar features. (Un-Solar, US)
             Then send each US image back to
             its original uncentered coordinates (uc).             *.b.fc.USuc*
             
          6) Multiply each *fcUSuc* 
             by its *dsk* image.  A *dsk* image is a disk with a
	     value NORM everywhere on the disk.  The positions of the
	     disk are the *un-centered* coordinates.
	     This step just eliminates and outer ring (R) from the
	     image.                                               *.b.fc.USucR*
	  
	  7) Average all the *dsk* images together.               *dsk_avg*
	     
          8) norm_accumulate  all *.b.fc.USucR* images to get     *resid*

	  9) norm_over the *resid* image by the *dsk_avg* 
	     image.   This leaves an *resid* image containing
	     unaccounted for, or "residual"  flat field features. *resid*

	 10) Create a new *paint* image by paint = paint*resid.

	     Write to disk the avg diameter found.
	     Write to disk this new paint image.

	 11) Subtract NORM from the *resid* image.
	     IF the sum of all the pixels in the difference is < 0.2
		***** we're done!
	     ELSE go back to step ***1*** above using the newly created *paint*
		  image.

*/
