/**************** findshift ***********************/

/*   Code to find x and y shift to "center" any "disk" type image.     */
/*   This is iterative, recalculating the centers until the difference */
/*       from the last calculation is less than 0.5 pixels.            */
/*                                 Alice Lecinski,  January 1996       */

/*   Pass this code --> img     -- your short image.
                       numx        the x-dimension of your "img"
                       numy        the y-dimension of your "img"
                       xshi        This will be found by findshift.
				       It is the x-shift from the center
				       in fractional pixels.
                       yshi        This will be found by findshift.
				       It is the y-shift from the center
				       in fractional pixels.
                       diam        An approximate first guess at the "diam"
				       *must* be passed to findshift.
				       The true diameter is returned.
*/
/******************************************************************************/

#include <stdio.h>
#define ILIMIT      (10)  /* Iterations before failure */
#define NSPOTS      (10)
#define LIMIT       (0.5)  /* pixel separation between previous and present
                              center computation. */
#define MNSOB    ( -500.) /* The sobel minimum must be less than this,
                             or it isn't a real edge. was -1500. */

float   ledge[NSPOTS], redge[NSPOTS], *le, *re;
float     bot[NSPOTS],   top[NSPOTS], *be, *te;



void findshift(img,numx,numy,xshi,yshi,diam)
    short *img;
    float *xshi,*yshi, *diam;
    int numx,numy;
{

    long ii,jj,kk,iter, strt;
    float xcen,ycen, sepx,sepy, XCEN,YCEN, goodx,goody, goodt,goodb;
    float lastx,lasty;
    float *ximg,*yimg,*sximg,*syimg;
    register float *qq, *pp, *ss;
    register short *iq;
    float sxmn,sxmx,symx,symn;
    float sobel[11]={ -6.,-2.,0.,2.,4.,4.,4.,2.,0.,-2.,-6.};
    int   hs=5, whs=11;
    int notgoodenough;
    float RE,LE,TE,BE;
    float xshift,yshift,t_diam;
    double yedge;
    double xdiff, ydiff, xydiff, fabs();
    int doprint=0; /* If doprint==1, findings will be printed
		      out during execution. */

    notgoodenough=1;
    iter=0;

    ximg  = (float *) malloc ( numx*     sizeof ( float )) ;
    sximg = (float *) malloc ( numx*     sizeof ( float )) ;
    yimg  = (float *) malloc ( numy*     sizeof ( float )) ;
    syimg = (float *) malloc ( numy*     sizeof ( float )) ;

    XCEN = xcen = lastx = (numx-1.)*0.5;
    YCEN = ycen = lasty = (numy-1.)*0.5;

    while(notgoodenough) {

        pp= sximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp=  ximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp= syimg; for(ii=0;ii<numy;ii++) *pp++=0.;
        pp=  yimg; for(ii=0;ii<numy;ii++) *pp++=0.;

        /* Find the left and right edges. */

        strt = (int)(ycen-NSPOTS*0.5);  /* Get to the "middle" of the image. */
        re=redge; le=ledge;
        for(kk=0;kk<NSPOTS;kk++){

            /**** Zero the X-sobel area, and grab a X row from the image. */

            pp= sximg;             for(ii=0;ii<numx;ii++) *pp++=0.;
            qq=  ximg; 
            iq=img+(strt+kk)*numx; for(ii=0;ii<numx;ii++) *qq++ = *iq++;


            /**** Apply the sobel to the X-line. */
            pp=sximg+hs;
            for(ii=hs;ii<numx-hs;ii++){
                ss=sobel;
                qq=ximg+ii-hs;
                for(jj=0;jj<whs;jj++) { *pp += (*ss) * (*qq) ; ss++; qq++; }
                pp++;
            }

            /**** Find the minimum (the edge) on the left           */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=sximg+whs;
            sxmn=0.;
            for(ii=0;ii<XCEN;ii++){ sxmn= (sxmn < *pp) ? sxmn : *pp; pp++; }
            ii=whs; pp=sximg+whs;   while( *pp++ > sxmn ) ii++;
                                    while( *pp++ <    0 ) ii++;
            *le++=(float)(ii-1) + (0.-*(pp-1))/((float)(*pp-*(pp-1)));

	    if(doprint)
		fprintf(stderr,"left %10.2f   sxmn=%10.2f\t",*(le-1), sxmn);
 
            /**** Find the minimum (the edge) on the right          */
            /**** Gotta go backwards.                               */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=sximg+numx-1-whs;
            sxmn=0.;
            for(ii=0;ii<XCEN;ii++){ sxmn= (sxmn < *pp) ? sxmn : *pp; pp--; }
            ii=numx-1-whs; pp=sximg+numx-1-whs;      while( *pp-- > sxmn ) ii--;
                                                     while( *pp-- <    0 ) ii--;
            *re++=(float)(ii) + (0.-*pp)/((float)(*(pp+1)-*(pp)));

	    if(doprint)
		fprintf(stderr,"right %10.2f  sxmn=%10.2f\n",*(re-1),sxmn);
        }





        /* Find the top and bottom edges. */

        strt = (int)(xcen-NSPOTS*0.5) ;
        be=bot; te=top;
        for(kk=0;kk<NSPOTS;kk++){

            /**** Zero the Y-sobel area, and grab a Y row from the image. */
            pp= syimg;      for(ii=0;ii<numy;ii++)  *pp++=0.;
            qq=  yimg; 
            iq=img+strt+kk; for(ii=0;ii<numy;ii++){ *qq = *iq; qq++; iq+=numx; }
            pp=syimg+hs;

            /**** Apply the sobel to the Y-line. */
            for(ii=hs;ii<numy-hs;ii++){ 
                ss=sobel;
                qq=yimg+ii-hs;
                for(jj=0;jj<whs;jj++) *pp += *ss++ * *qq++ ;
                pp++;
            }

	    /**** Find the minimum (the edge) on the bottom         */
	    /**** Then find the zero crossing as a fracional pixel. */
            pp=syimg+whs;
            symn= MNSOB;  /*If it isn't less than this, it isn't a real edge*/
            for(ii=0;ii<YCEN;ii++){ symn=(symn < *pp) ? symn : *pp; pp++; }
            if (symn < MNSOB)  {
                ii=whs; pp=syimg+whs;   while( *pp++ > symn ) ii++;
                                        while( *pp++ <    0 ) ii++;
                *be++=(float)(ii-1) + (0.-*(pp-1))/((float)(*pp-*(pp-1)));
            }
            else *be++ = numy;

	    if(doprint)
		fprintf(stderr,"bottom %10.2f  symn=%10.2f\t",*(be-1),symn);
 
	    /**** Find the minimum (the edge) on the top            */
	    /**** Gotta go backwards.                               */
	    /**** Then find the zero crossing as a fracional pixel. */
            pp=syimg+numy-1-whs;
            symn= MNSOB;
            for(ii=0;ii<YCEN;ii++){ symn=(symn < *pp) ? symn : *pp; pp--; }
            if (symn < MNSOB)  {
                ii=numy-1-whs; pp=syimg+numy-1-whs;  while( *pp-- > symn ) ii--;
                                                     while( *pp-- <    0 ) ii--;
                *te++= (float)(ii) + (0.-*pp)/((float)(*(pp+1)-*(pp)));
            }
            else *te++ = 0.;

	    if(doprint)
		fprintf(stderr,"top %10.2f   symn=%10.2f\n",*(te-1),symn);
        }

        /*  Check the separations between the top and bottom
            and the right and left edges.  The separations should be close 
            to the known diameter of the sun for the instrument.  If not
            the edge was found incorrectly.  In particular, this happens
            for the bottom and top edges when the image of the sun is 
            moved around in the fov to find the flat field. 
        */
        RE=LE=BE=TE=0.; 
        re=redge; le=ledge; be=bot; te=top;

	if(doprint)
	    fprintf(stderr,"\n");

        goodx=goody=0.;
        for (ii=0;ii<NSPOTS;ii++){

	    if(doprint)
               fprintf(stderr,"%10.2f %10.2f\t%10.2f %10.2f\n",*le,*re,*be,*te);

            sepx= *re - *le;
            if(sepx < (*diam)+0.36*(*diam) && sepx > (*diam)-0.36*(*diam)) {
                LE += *le++ ; RE += *re++ ;
                goodx++;
            }
            sepy= *te - *be;
            if(sepy < (*diam)+0.36*(*diam) && sepy > (*diam)-0.36*(*diam)) {
                BE += *be++ ; TE += *te++ ;
                goody++;
            }
        }
        LE /= goodx; RE /= goodx; BE /= goody; TE /= goody;
        xcen  = (RE-LE)*0.5 +LE;
        ycen  = (TE-BE)*0.5 +BE;
                         
                        /* If the y separations were hideous, go back and */
        if(goody==0) {  /* find a good top **or** bottom edge. */
            BE=TE=0.; 
            goodt=goodb=0.;
            be=bot; te=top;
            for (ii=0;ii<NSPOTS;ii++){
                if((*be>whs ) && (*be<YCEN)    ) { BE += *be++ ; goodb++; }
                if((*te>YCEN) && (*te<numy-whs)) { TE += *te++ ; goodt++; }
            }
            if      (goodb>0.) { 
                BE /= goodb; yedge=(double)BE; BE=0.; 
                goodb=0.;
                be=bot;
                for (ii=0;ii<NSPOTS;ii++){
                    ydiff=yedge- (double)*be;
                    if( fabs(ydiff) < 0.1*numy ) { BE += *be++ ; goodb++; }
                }
                BE /= goodb;  TE=BE+(*diam);  ycen=BE+0.5*(*diam);
            }
            else if (goodt>0.) {
                TE /= goodt; yedge=(double)TE; TE=0.; 
                goodt=0.;
                te=top;
                for (ii=0;ii<NSPOTS;ii++){
                    ydiff=yedge- (double)*te;
                    if( fabs(ydiff) < 0.1*numy ) { TE += *te++ ; goodt++; }
                }
                TE /= goodt;  BE=TE-(*diam);  ycen=TE-0.5*(*diam);
            }
            else {
                fprintf(stderr,"Neither a good top or bottom was found.\n");
                TE=YCEN+(*diam)*0.5 + 2.*LIMIT*(float)iter;
                BE=YCEN-(*diam)*0.5 + 2.*LIMIT*(float)iter;
                ycen  = (TE-BE)*0.5 +BE;
            }
        }

        t_diam= ((RE-LE) + (TE-BE))*0.5;
        *xshi = xshift = XCEN-xcen;
        *yshi = yshift = YCEN-ycen;

	if(doprint) {
	    fprintf(stderr,"Iteration: %d\n",++iter);
	    fprintf(stderr,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
	    fprintf(stderr,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
	    fprintf(stderr,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
	}
      

        /* Check to see how close the centers are to the previous iteration.
           Note, the while loop will happen at least twice...
        */
        xdiff = lastx-xcen;  xdiff = fabs(xdiff);
        ydiff = lasty-ycen;  ydiff = fabs(ydiff);
        xydiff = xdiff + ydiff;
        if (xydiff < LIMIT) notgoodenough=0;
        if (iter   > ILIMIT) exit(-20);

        lastx=xcen; lasty=ycen;

    }

    *diam = t_diam;

    if(doprint) {
	fprintf(stderr,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
	fprintf(stderr,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
	fprintf(stderr,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
    }

    free( (char *) ximg );
    free( (char *)sximg );
    free( (char *) yimg );
    free( (char *)syimg );
}
