/* Read README.calibrate
*/
#include                <stdio.h>
#include                <stdlib.h>
#include                <math.h>
#include                <errno.h>
#include                <fcntl.h>

#define START
#include                "dim.h"
#include                "shift.h"

#include                "acos.h"
#include                "acos_header.h"

#include                "params.h"

#define SLN      120        /* string length of filenames. */


short   paint[IMSZ],  bxr[IMSZ],  img[IMSZ];
float  fpaint[IMSZ], fbxr[IMSZ], fimg[IMSZ], fwork[IMSZ], spaint[IMSZ];
float  *s_p ,      *s_bxr;  /* the shifted paint and bxr images. */

char  (*ff)[SLN],  files[1100][SLN];
char  biasfile [SLN];
char  accumfile[SLN];
char  paintfile[SLN];
char      afile[SLN];

struct acos_header acos_h, *ah;

main(an,av) int an; char *av[];
{
    int   ii,jj,kk;
    int   numimgs=0;
    int   gotbias=0, gotpaint=0, do_rot=0;
    char  *w, *ww;
    short *ip,*jp,*kp;
    float *fp,ovsum,sum,normsum,normal,bsum,ovbsum;
    float ovnum;
    float pangle=0., diam=0. , avgdiam;
    float xs,ys;

    void writefvalue();
    void fdoshift();
    FILE  *fid;

    nrow = NROW;
    ncol = NCOL;
    imsz = IMSZ;
    im_nbytes=sizeof(short)*imsz;

    sprintf(accumfile,"accumlist");

    ff = files;
    ah = &acos_h;

    if ( an <= 1 ) { prsum() ; exit(3) ; }
 
    while (--an) { /* Read in any arguments, and the image filenames. */
        w = *++av ;
        if (*w == '?') { prsum(); exit(2); }
        if (*w == '-') { /* Check for flag arguements. */
            switch (w[1]) {
		case 'b':   --an ; w = *++av;
			    sscanf(w,"%s",biasfile) ;
			    gotbias=1;
			    break;
		case 'p':   --an ; w = *++av;
			    sscanf(w,"%s",paintfile) ;
			    gotpaint=1;
			    break;
		case 'r':   /* Rotate to heliocentric coordinates. */
			    do_rot=1;
			    break;
		default :
                            fprintf (stderr," unknown flag %s\n",*av);
                            prsum(); exit(1);
            }
	}
	else {
	    sprintf(*ff++,"%s",w);
	    numimgs++ ;
	}
    }
    if ( gotbias+gotpaint != 2 ) { prsum(); exit(3); }

    ovnum = 1./ (float)numimgs;

    /* Read and copy to floats the bias and paint files.
    */
    readimg(biasfile ,bxr);    rereadhdr(bxr,ah);    bsum=(float)(ah->Nsum);
						     bsum = (bsum<=0) ? 1:bsum;
						   ovbsum= 1./bsum;
						     s2f(bxr,fbxr);
						     fvtimes(fbxr,ovbsum);
    readimg(paintfile,paint);                        s2f(paint,fpaint);
						     normal = OVNO ;
			                             fvtimes(fpaint,normal);

    ff=files;
    ah = &acos_h;
    for(ii=0;ii<numimgs;ii++) {
	readimg(*ff,img); rereadhdr(img,ah); zhdr(img); s2f(img,fimg);

	sum = (float)(ah->Nsum); ovsum = 1./sum ;

	/* Handle the bias. */
	for(jj=0;jj<ah->Nsum;jj++) {
	    s_bxr=fwork; 
	    xs = ah->Xshift[jj];
	    ys = ah->Yshift[jj];
	    fdoshift(fbxr,s_bxr,ncol,nrow,xs,ys);
	    fminus(fimg,s_bxr);
	}

	/* Handle the fixed bias, which is a sag in the vertical direction
	   in images taken with Megaplus. */
	ffixbias(fimg,ah);

	/* Handle the paint.  
	   Have to sum the paint values at the proper shift positions,
	   and then, at the very end, divide the image by the summed paint. */
	fzero(spaint);
	for(jj=0;jj<ah->Nsum;jj++) {
	    s_p=fwork; 
	    xs = ah->Xshift[jj];
	    ys = ah->Yshift[jj];
	    fdoshift(fpaint,s_p,ncol,nrow,xs,ys);
	    fplus(spaint,s_p);
	}
	normal = 1./((float)(ah->Nsum));
        fvtimes(spaint,normal);
	fover(fimg,spaint);

	/* Normalize the images to a sum of 8 images. */
	normal = 8./((float)(ah->Nsum));
        fvtimes(fimg,normal);

	/* Normalize the images to a constant area of a sun of radius 960". */
	/*
	normal = (960.*960.)/(ah->Radius * ah->Radius);
        fvtimes(fimg,normal);
	*/


	avgdiam=0.;
	for(jj=0;jj<ah->Nsum;jj++) {    diam += ((ah->Diam[jj])*ovsum);
				     avgdiam += ((ah->Diam[jj])*ovsum); }
	pangle +=  ah->Pangle;


	f2s(fimg,img);
	zero_shifts(img, ah);
	if(do_rot ){ do_rotate  (img, ah->Pangle);
	               flipTB   (img,ncol,nrow); }


	ah->Type             = (ah->Type / 10) *10  + REDUCTION_c ;
	if(do_rot ) ah->Type = (ah->Type / 10) *10  + REDUCTION_h ;

	NameChip(afile,ah);

	if(do_rot ) sprintf(afile,"%s.bfbpr",*ff);

	replacehdr(img,ah);
	writeimg(afile,img);

	ff++;
    }
    /* Save the diameter and pangle. */
    /*
    diam   *= ovnum; sprintf(afile,"diam");   writefvalue(afile,diam);
    pangle *= ovnum; sprintf(afile,"pangle"); writefvalue(afile,pangle);
    */
    exit(0);

}

int
prsum() {
    static char help[] = "\
chip_bfbp [-r] [-b biasfile] [-p paintfile] [file1] [file2] ... [fileN]\n\
\n\
Example: chip_bfbp -b bxr -p hfr.paint 96d043.210248.hsr 96d043.210317.hsr\n\
\n\
Example: chip_bfbp -b bxr -p cfr.paint 96d043.210249.csr 96d043.210318.csr\n\
\n\
This code calibrates helium and continuum chip images using a\n\
     bias file and a \"flat\" or \"gain\" or \"paint\" image.\n\
     The fixbias is also removed.\n\
The output files are: \"??????.hsc\" \n\
The output files are: \"??????.csc\" \n\
\n\
Flags       Description\n\
-----       -----------\n\
 [-r]       If specified, images will be rotated to *heliocentric*\n\
		   coordinates.\n\
\n\
";

    fputs (help,stderr);

}
