/* Read   "README_getchipflat.h"
   Read   "USAGE_getchipflat.h"
*/
#include                <stdio.h>
#include                <stdlib.h>
#include                <math.h>
#include                <errno.h>
#include                <fcntl.h>


#include                "README_getchipflat.h"
#include                "USAGE_getchipflat.h"

#define START
#include                "dim.h"
#include                "shift.h"
#include                "params.h"

#define TLIMIT  (  0.5)     /* Was 0.2 -- changed 1/29/97 --Alice */
#define DELDIAM (-50.0)
#define SLN      120        /* string length of filenames. */
#define PAD      (20)

#define CDIAM   (840.)      /* an approximate solar diameter in pixels. */

short *solar, *resid, *dsk_avg, *hbr, *dsk;

short paint[IMSZ], img[IMSZ],      work[IMSZ], avg[IMSZ] ;
float fwork[IMSZ], fdsk_avg[IMSZ];

char  (*ff)[SLN],  files[50][SLN], /* 50 possible files of */
      (*wf)[SLN], wfiles[50][SLN]; /* strlen(SLN)           */
char  biasfile [SLN];
char  accumfile[SLN];
char  paintfile[SLN], shiftfile[SLN], tempfile[SLN];

char  rmcmnd[200];
char  ctemp[20];
char  fmt  [80];

main(an,av) int an; char *av[];
{
    int   ii,jj,kk;
    int   numimgs=0, numiter=1,dofast=0;  /*  dofast=1 takes advantage of
					      the fact that the radii and
					      shifts of the images change
					      only very slightly from the 
					      first iteration.  Therefor the
					      dsk images and avg of the
					      dsk images need only be
					      calculated once.

					      It does **NOT** give as
					      good results as when
					      dofast=0.
					  */

    int   saveimages=0,rmimages=1,doprint=0;  /* saveimages=1 saves various
						 working files along the way.
						 Set saveimages=1 if you want
						 to double check how things
						 work. */
    int   tryagain=1;
    int   xtr;
    int   Hour;
    char  *w, *ww;
    short *ip,*jp,*kp;
    float *fp,ovnum,del_diam;
    float  rndr=0.5005;    /* Value to round up to nearest integer. */
    double tot, val, hlimit,llimit,npts,fabs();
    FILE  *pfid;
    SHFT  xyshft,shft;

    nrow = NROW;
    ncol = NCOL;
    imsz = IMSZ;
    im_nbytes=sizeof(short)*imsz;

    del_diam=DELDIAM;
    xtr = PAD;

    sprintf(biasfile, "hbr");         /* The bias image. */
    sprintf(accumfile,"accumlist");   /* A file to list files to add together.*/
    sprintf(shiftfile,"hfr.diam");    /* A file to record the solar diameter. */

    ff = files;

    if ( an <= 1 ) { prsum() ; exit(3) ; }
 
    while (--an) { /* Read in any arguments, and make a list of the files. */
        w = *++av ;
        if (*w == '?') { prsum(); exit(2); }
        if (*w == '-') { /* Check for flag arguments. */
            switch (w[1]) {
		case 'b':   --an ; w = *++av;
			    sscanf(w,"%s",biasfile) ;
			    sscanf(w,"%7.7s%2.2d",ctemp,&Hour);
			    break;
		default :
                            fprintf (stderr," unknown flag %s\n",*av);
                            prsum(); exit(1);
            }
            continue;
       }
       sprintf(*ff++,"%s",w);    numimgs++;
    }
    ovnum=1./(float)numimgs;

    w = (char *)files;  
    sprintf(fmt,"%%%d.%dspt",strlen(files[0])-2,strlen(files[0])-2);
    sprintf(paintfile,fmt,w);
    if ( *(w+strlen(files[0])-3) == 'c' ) {  /* Check and change filename
						if doing continuum. */
	sprintf(shiftfile,"cfr.diam");
    }
    fprintf(stderr,"Working to create %s\n",paintfile);

    /* Create an initial "paint" image of constant level NORM */
    ip=paint; for(ii=0;ii<imsz;ii++) *ip++ = INORM;

    xyshft = (SHFT)malloc(sizeof(struct shifts)*numimgs);/* Space for center  */
							 /* positions (shifts */
							 /* from it) and      */
							 /* diameters. */
    /* Initialize the shifts and diameters. */
    shft=xyshft;
    for(ii=0;ii<numimgs;ii++){ shft->X=shft->Y=0.; shft->Dia = CDIAM; shft++;}



    /*  First bias correct each image. */
    if(doprint)
	fprintf(stderr,"    Bias correction...\n");
    hbr=work;
    readimg(biasfile,hbr);
    ff=files;
    for(ii=0;ii<numimgs;ii++){
	readimg(*ff,img);
	minus(img,hbr);
	sprintf(*ff,"%s.b",*ff); is_fits=0;
	writeimg(*ff,img);
	ff++;
    }

    /*  Begin the iteration to find the flat. Criteria that must be met is that
        the sum of (residual solar and instrument inconsistancies) < TLIMIT
    */
    while(tryagain) {

	fprintf(stderr,"\tIteration %02d\n",numiter);
	if(numiter > 10) {
	    fprintf(stderr,
	    "Data is not converging, giving up, removing paint.\n");
	    sprintf(rmcmnd,"/bin/rm %s\n",paintfile);
	    pfid=popen(rmcmnd,"r+"); ii=pclose(pfid);

	    sprintf(rmcmnd,"/bin/rm *.b *.fc *.USucR\n");
	    pfid=popen(rmcmnd,"r+"); ii=pclose(pfid);

	    exit(-20);
	}


	/*  Flatten and center each bias corrected image.  Makes "fc" images. */
	if(doprint)
	    fprintf(stderr,
	    "\t    Flatten and center each bias corrected image.\n");
	/* This step should remove the flat field - aka the instrument field. */
	ff=files; wf=wfiles; shft=xyshft;
	for(ii=0;ii<numimgs;ii++){
	    readimg(*ff,img);
	    over(img,paint);  /* Flatten the image, ie, remove the flatfield. */
	    /* Find the shifts from center. */
	    if(dofast) {
		if(numiter==1)
		findshift(img,      ncol,nrow,&shft->X,&shft->Y,&shft->Dia);
	    }
	    else       {
		findshift(img,      ncol,nrow,&shft->X,&shft->Y,&shft->Dia);
	    }
	    /* Center each image. */
	    do_shift     (img,fwork,ncol,nrow, shft->X, shft->Y, shft->Dia);
	    sprintf(*wf,"%s.fc",*ff); is_fits=0;
	    writeimg(*wf,img);         /* Write out the "fc" images.          */
	    shft++; ff++; wf++;
	}

	/* Create a "solar" image which
	   is the average of the above "fc" images, normalized to NORM.
	   Since the "fc" images have had the flatfield removed, their
	   average, "solar" should be predominated by solar features.
	*/
	solar=avg;
	writelist                (accumfile,wfiles,numimgs,SLN);
	norm_accumulateSIGMALIMIT(accumfile,solar,2.0);

	/*  Divide each "fc" image, by the "solar" image.
	    This should remove the solar features  hence -> Un-Solar, or **US**
	    Then send each US image back to its original uncentered 
	    coordinates *uc*  --> USuc
	    Then multiply each USuc image by a *disk* image which
	    removes a ring of del_diam from the image *R* --> USucR
	*/
	if(doprint)
	    fprintf(stderr,"\t    Make USucR images.\n");
	if( dofast ) {    
	    /* If dofast, only zero out fdsk_avg on the first iteration.
	       fdsk_avg holds the sum of all of the uncentered dsk images.
	    */
	    if(numiter==1) { fp=fdsk_avg; for(ii=0;ii<imsz;ii++) *fp++ = 0.; }
        }
	else {               fp=fdsk_avg; for(ii=0;ii<imsz;ii++) *fp++ = 0.; }
	wf=wfiles; shft=xyshft; dsk=work;
	for(ii=0;ii<numimgs;ii++){
	    readimg(*wf,img);
	    over(img,solar);
	    if(dofast){ /*  See if this is faster... */
		un_shift (img ,fwork,ncol,nrow, shft->X, shft->Y, shft->Dia);
		if(numiter==1) {
		    disk (dsk ,       del_diam, shft->X, shft->Y, shft->Dia);
		    /* Save the dsk file for subsequent iterations. */
		    sprintf( tempfile,"%s.dsk",*wf); is_fits=0;
		    writeimg(tempfile,dsk);
		    /* Make the sum of the dsk files. */
		    fp=fdsk_avg; ip=dsk ;
		    for(jj=0;jj<imsz;jj++) { *fp += (float)(*ip); fp++;ip++; }
		}
		else {
		    sprintf(tempfile,"%s.dsk",*wf);
		    readimg(tempfile,dsk);
		}
		times(img,dsk );
	    }
	    else { /* Slow way? */
		un_shift (img ,fwork,ncol,nrow, shft->X, shft->Y, shft->Dia);
		disk     (dsk ,       del_diam, shft->X, shft->Y, shft->Dia);
		times(img,dsk );
		fp=fdsk_avg; ip=dsk ;
		for(jj=0;jj<imsz;jj++) { *fp += (float)(*ip); fp++;ip++; }
	    }


	    sprintf(*wf,"%s.USucR",*wf); is_fits=0;
	    writeimg(*wf,img);
	    shft++; wf++;

	}

	fp=fdsk_avg; ip=dsk_avg=work;
	for(jj=0;jj<imsz;jj++) { *ip = (short)(rndr+(*fp)*ovnum); ip++; fp++;}

	if(saveimages) writeimg("dsk_avg",dsk_avg);

	/* Create a "resid" image which
	   is the average of the above "fc.USucR" images, normalized to NORM.
	   We'll have to divide by the right number of images used later below.
	*/
	resid=avg;
	writelist                (accumfile,wfiles,numimgs,SLN);
	norm_accumulateSIGMALIMIT(accumfile,resid,2.0);

	if(saveimages) writeimg("accum_resid",resid);

	norm_over(resid,dsk_avg);  /* After this, any residual flat field or
	 			      instrument inconsistancies are
				      stored in resid
				   */
	if(saveimages) writeimg("residual",resid);

	/* Fix up the paint image by multiplying by the "residual".
	   resid is the "residual", 
	   paint is the previous paint, or flatfield
	   After "times" paint=    paint*resid
	*/
	times(paint,resid);
	writeimg(paintfile,paint);
	writeshft_diam(shiftfile,files,xyshft,numimgs,SLN);


	/* Now, check to see if the residuals are small enough.  If so,
	   we can stop.
	*/
	tot=0.; npts=0.;           hlimit=NORM+400.; llimit=NORM-400.;
        ip=resid; 
	for(ii=0;ii<imsz;ii++) {   /* Don't count the bad pixels.... */
	    val = (double)(*ip); 
	    if((val < hlimit) && (val > llimit)) { tot += (NORM-val);  npts++; }
	    ip++;
	}
	tot /= npts;

	fprintf(stderr,"\t\tAverage residual deviation %6.2f    Tossed %.0f\n",
			  tot,(double)imsz-npts);

	tryagain = (fabs(tot) >= TLIMIT) ? 1
					 : 0 ;
	if(numiter<=2)tryagain=1;

	numiter++;
    }

    fprintf(stderr,
    "\t\t\tAll done with  %s\n\
    Hour  %d \n\
    Final Iterations %d \n\
    Final Deviation  %6.2f\n\n",paintfile,Hour,numiter,tot);

    if(rmimages) {
        sprintf(rmcmnd,"/bin/rm *.b *.fc *.USucR\n");
	pfid=popen(rmcmnd,"r+"); ii=pclose(pfid);
    }

}

prsum() {
    static char help[] = "\
getchipflat [-b biasfile] [flatfile1] [flatfile2] ... [flatfileN]\n\
\n\
Example: getchipflat -b 95d265.200113.hbr 95d265.203044.hfr 95d265.203205.hfr 95d265.203355.hfr\n\
\n\
Example: getchipflat -b 95d265.200113.hbr 95d265.203102.cfr 95d265.203220.cfr 95d265.203330.cfr\n\
\n\
This code creates a \"paint\" image to calibrate helium or\n\
	  continuum images.\n\
The output file is: \".cpt\" or \".hpt\"\n\
\n\
Flags          Description\n\
-----          -----------\n\
\n\
";

    fputs (help,stderr);
}
