/*
		limb_p diam del_diam >limb_file

Version of Tue May 14 09:36:25 MDT 1996

		limb_p uses P angle to correct for
		helium image solar rotation.

		diam is a file containing an ascii floating point diameter
		and the P angle in degrees.
		Diameter is in pixels.
		For chip, use the header value for radius in arc
		seconds, multiply by 2, and divide by 2.29.

		del_diam is a floating point constant to be added
		to the value of diam.  Currently this is set to
		+0.

		This routine generates a limb darkened disk
		of diameter specified in the call.
		The intensity is normalized to 8192.

		The limb profile uses the formula given in Allen
		and the constants are set for 1 micron.

		There is also a rotation induced trend removed from
		the limb profile, over the region occupied by
		the disk.

 This is the expression from Allen

 I(theta)/I(0) = a + b cos(theta)+c[1-cos(theta)ln(1+1/cos(theta))]
 where a+b+(1-ln(2)*c = 1 or
 a + b + .306853 * c = 1

*/


#include		<stdio.h>
#include		<stdlib.h>
#include		<errno.h>
#include		<math.h>
#include		"dim.h"

main(argc,argv)
char		**argv;
int		argc;
{
/* this is the fractional rotation-induced trend constant */
/* One side of image center this fraction is added per pixel */
/* from the center location.  On the other side, that fraction */
/* is subtracted. */

float		rotscale = .023;
/* These are the limb darkening constants used in the formula */
/* in Allen Astrophysical Quantities */

float		a = 1.09;    /* Bigger rolls up limb */
float		b = .143; 	/* Bigger puts dip in center of disk */

/* Use limb_p with +0, bigger causes more roll off */
/* 1.20 and 0.095 with limb +7 is very S shaped */
/* 1.10 and .139 with limg +2 looks a little S shaped with dark moat */
/* 1.10 and .143 with limb +4 looks good */
/* 1.01 and .185 with limb +1 is pretty flat */
/* .90  and .23  with limb +0 both rolls off and spikes at limb */
/* .97 and .20 used initially for CHIP display */
/* .80 and .30 dark in the middle *?
/* .60 and .38 is flat with roll off toto */
/* .95 and .20 is flat and goes closer to limb */
/* 1.18 and 0.10 is flat and goes very close to limb ,maybe a little
roll off too soon*/
/* 1.08 and 0.15 looks good */
/* 1.35 and 0.00 is bright in the middle */
/* 1.06 and 0.15 work with limb reduced by 5 pixels */
/* might want to bring down center a little */

char		*p;

float		c;

FILE		*diamfile;
int			isafile,i,j,code,write();
short		*img;
float		ctheta,theta;
float		deldiam,fi,fj,pow,diam,val,x,y,xcen,ycen,rad,rvec,pangle;
float		delx,dely;
register	short	*ip;

int			nrow,ncol,record;

sscanf ( getenv ( "XDIM" ),"%d",&ncol );
sscanf ( getenv ( "YDIM" ),"%d",&nrow );
record = 2*ncol;
if ( ncol == 0 ) 
	{
	ncol = NCOL;
	nrow = NROW;
	record = RECORD;
	}
img = (short*)malloc ( (size_t)(2*ncol*nrow) );
/*fprintf ( stderr,"%d %d\n",ncol,nrow );*/


c = (1. - a - b )/.306853;

deldiam = 0.;
isafile = 0;
if ( argc > 1 )
	{
	p = *++argv;
	if ( ( *p != '+' ) && ( *p != '-' ) )
		{
		isafile = 1;
		if ( ( diamfile = fopen ( p,"r" ) ) < (FILE *)0 )
			{
			fprintf ( stderr," open error %d %s\n",diamfile,p );
			goto usage;
			}
		if ( ( code = fscanf ( diamfile,"%f %f",&diam,&pangle ) ) < 0 )
			{
			if ( code == -1 )
				{
				fprintf ( stderr," read error %d %s\n",code,p );
				goto usage;
				}
			}
		}
	else 
		{
		sscanf ( p,"%f",&diam );
		p = *++argv;
		sscanf ( p,"%f",&pangle );
		}
	if ( argc > 2 )
		{
		p = *++argv;
		sscanf ( p,"%f",&deldiam );
		}
	}
else
	{
usage:	fprintf ( stderr,"usage: limb arg  (arg is a file or +/-const.)\n" );
		exit ( 1 );
	}

pangle = -.01745*pangle;
delx = rotscale*cos(pangle);
dely = -rotscale*sin(pangle);
rad = (diam+deldiam)/2.;

xcen = (ncol-1)/2.;
ycen = (nrow-1)/2.;

ip = img;

for ( i=0; i<nrow ; i++ )
	{
	fi = i;
	y = (fi-ycen)/rad;
	for ( j=0 ; j<ncol ; j++ )
		{
		fj = j;
		x = (fj-xcen)/rad;
		rvec = sqrt(x*x + y*y);
		if ( rvec >= 1. )
			*ip++ = 3000.;
		else
			{
			theta = asin(rvec);
			ctheta = cos(theta);
			val = a;
			val = val + b*ctheta;
			val = val + c*(1-ctheta*log(1.+1./ctheta));
/* this is an ad hoc solar rotation term */
			val = val * (1. + delx*x );
			val = val * (1. + dely*y );
			val = val * 8192.;
			*ip++ = (short)val;
			}
		}
	}

if ( ( code = write ( 1,img,record*nrow ) ) != record*nrow )
	{
	fprintf ( stderr," write error %d \n",code );
	exit ( 1 );
	}
}

