/******************************************************************************
    Name:    mechanism.h

    Purpose:    Define the structure used to access hardware 
        drive and status registers.
        Specify the offsets and masks for each device
        Routines checkStatus, checkDrive, and setDrive
        use this structure.

    Author:    David Elmore

    Date:     Mon Aug 21 09:54:46 MDT 1995

******************************************************************************/

#include    <mechanism_defs.h>

/* Mechanism associates bits in the interfaces to device numbers */
/* This is used to shift mechanism interface bit positions to the low */
/* order bits in a character so that they make sense */

struct    mechanism {
        int    stat_byte;        /* Byte number in VMIC cci interface */
        int    stat_mask;        /* Bits containing this device in byte */
        int    stat_shift;        /* How far left shifted the mask is */
        int    drive_byte;        /* Byte number in VMIC ddo interface */
        int    drive_mask;        /* Bits containing this device in byte */
        int    drive_shift;    /* How far left shifted the mask is */
        };

#ifdef START

struct    mechanism    mech[NUM_DEV] = {
    0,    0x03,    0,    0,    0x03,    0,    
    0,    0x0c,    2,    0,    0x0c,    2,    
    0,    0x30,    4,    0,    0x30,    4,    
    0,    0xc0,    6,    0,    0xc0,    6,    

    1,    0x03,    0,    1,    0x03,    0,    
    1,    0x03,    0,    1,    0x03,    0,    
    1,    0x0c,    2,    1,    0x0c,    2,    
    1,    0x30,    4,    1,    0x30,    4,    
    1,    0xc0,    6,    1,    0xc0,    6,    

    2,    0x07,    0,    2,    0x03,    0,    
    2,    0x70,    4,    2,    0x30,    4,    

    3,    0x07,    0,    3,    0x03,    0,    
    3,    0x70,    4,    3,    0x30,    4    
    };

int    byte_mechs[NUM_BYTE] = {4,5,2,2};
int    first_mech[NUM_BYTE] = {0,4,9,11};

int    filter_lcvr[4] = {0,1,2,3};
int    mod_lcvr = 7;
int    In = IN;
int    Out = OUT;
int    Diffuser = CHIP_DIFFUSER;
int    Cover = CHIP_COVER;
#else

extern    struct    mechanism mech[NUM_DEV];
extern    int    byte_mechs[NUM_BYTE];
extern    int    first_mech[NUM_BYTE];
extern    int    filter_lcvr[4];
extern    int    mod_lcvr;
extern    int    In,Out,Diffuser,Cover;

#endif

