/* N_hsrMcsr7.c
 * -----------------------------------------------------------------------------
 * Author: Alice Lecinski   HAO/NCAR   8 Mar 2000
 * Updates: Andrew L. Stanger
 * 17 Jul 2002 : cosmetic changes only.
 * -----------------------------------------------------------------------------
 */

#include                <stdio.h>
#include                <stdlib.h>
#include                <math.h>
#include                <errno.h>
#include                <fcntl.h>

#define START
#include                "dim.h"
#include                "shift.h"

#include                "acos.h"
#include                "acos_header.h"

#include                "params.h"
#include                "annuli.h"
#include                "LDmodel.h"

#define SLN      120     /* string length of filenames. */


short   himg [IMSZ], cimg [IMSZ];
float  fhimg [IMSZ], fcimg [IMSZ], fdimg [IMSZ];

int    sunidx [NANNULI*NANN_PIXELS];   /* Index positions of a solar disk of
                                        * continuum radius to be used to
                                        * compute Helium index and Continuum
                                        * index for the image in question.
                                        */

char  *ff, Hf [SLN], Cf [SLN], afile [SLN];
char  clist [SLN], hlist [SLN];
char  ciout [SLN], hiout [SLN];

struct acos_header   acos_h, *ah;


main (an, av)

int an;
char *av [];
{
    int   ii, jj, kk;
    int   Hims = 0, Cims = 0, mkflat = 0, do_rot = 0, overrideRADIUS = 0;
    int   calcBCC = 1, calcBCH = 1;
    int   calcIndex = 1;
    int   doprint = 0;
    int   Nsunpx ;
    char  *w, *ww;
    float BCC = 1.0, BCH = 1.0, ovBCC, ovBCH;
    float HavgR = 0.0, CavgR = 0.0;
    float inval = 1.0, outval = -5.0;
    float sclfactor = 8192.0;
    float ovsunpx;
    float solve_LD_curve(), SolarIndex();
    double sunrad;
    float  air_factor;
    double HA;
    struct LDmodel_info  *LD;
    FILE  *hfid, *cfid  ,  *hifid, *cifid;


    nrow = NROW;
    ncol = NCOL;
    imsz = IMSZ;
    im_nbytes = sizeof (short) * imsz;

    ah  = &acos_h;
    LD  = &LDmodel;
    sprintf (clist, "csrlist");
    sprintf (hlist, "hsrlist");
    sprintf (ciout, "cont_index");
    sprintf (hiout, "heli_index");

    if ( an <= 1 ) { prsum () ; exit (3) ; }
 
    while (--an)
	{		/* Read in any arguments, and the image filenames. */
        w = *++av ;
        if (*w == '?') { prsum (); exit (2); }
        if (*w == '-')
	    {				/* Check for flag arguments. */
            switch (w [1])
		{
                case 'f':   mkflat=1;
                            break;
                case 'r':   do_rot=1;
                            break;
                case 'C':   --an ; w = *++av;
                            sscanf (w, "%s", clist) ;
                            break;
                case 'H':   --an ; w = *++av;
                            sscanf (w, "%s", hlist) ;
                            break;
                case 'B':   /* BCC  or BCH */
                            switch (w [3])
				{
                                case 'C':   calcBCC = 0; --an ; w = *++av;
                                            ii = sscanf (w, "%f", &BCC) ;
                                            if (ii != 1)
						{
                                                fprintf (stderr, 
                                                "Bad BCC entered:\t %s\n\n", w);
                                                prhelp (); exit (3);
                                                }
                                            break;
                                case 'H':   calcBCH = 0; --an ; w = *++av;
                                            ii = sscanf (w, "%f", &BCH) ;
                                            if (ii != 1)
						{
                                                fprintf (stderr, 
                                                "Bad BCH entered:\t %s\n\n", w);
                                                prhelp (); exit (3);
                                                }
                                            break;
                                default :   fprintf (stderr, 
                                                " unknown flag %s\n", w);
                                            prsum (); exit (1);
                                }
                case 'R':   overrideRADIUS = 1;
                            --an ; w = *++av;
                            sscanf (w, "%lf", &sunrad) ;
                            break;
                default :
                            fprintf (stderr, " unknown flag %s\n", *av);
                            prsum (); exit (1);
                }
            }
        }

    initLD (LD, BCC, BCH);

    /* Open the file lists, count the images from csr and hsr lists
     * make sure the numbers match, and save the avg_radius --
     * and make a disk image  to enhance off limb features....
     */

    if (! (cfid = fopen (clist, "r")) )
	{ fprintf (stderr, "%s does not exist.\n", clist); exit (1); }

    if (! (hfid = fopen (hlist, "r")) )
	{ fprintf (stderr, "%s does not exist.\n", hlist); exit (1); }

    /* Count number of images in each list. */
    /* Sum the radii for each list. */

    while (  (ii = fscanf (hfid, "%s", Hf)) == 1 )
	{
        Hims++;				/* Helium image counter. */
        if ( (ii = fscanf (cfid, "%s", Cf)) != 1 ) break;
        Cims++;				/* Continuum image counter. */
        readhdr (Hf, ah);
	HavgR += ah->Radius ;		/* arcsec */
        readhdr (Cf, ah);
	CavgR += ah->Radius ;		/* arcsec */
        }

    if (Hims != Cims)
	{
        fprintf (stderr, 
        "Number of hsr images does not equal number of csr images %d != %d \n", 
        Hims, Cims);
        exit (-20);
        }

    if (overrideRADIUS == 0)
	{
        HavgR /= ((float) Hims * 2.29 );	/* 2.29 pixels/arcsec */
        CavgR /= ((float) Cims * 2.29 );	/* 2.29 pixels/arcsec */
        sunrad = (double) HavgR ; 
        }

    fprintf (stdout, "Setting up annuli, this takes a while...\n");
    setup_annuli (CavgR, ah->Xdim, ah->Ydim);

    if (calcIndex == 1)
	{
        Nsunpx   = get_annuli_idx (sunidx, 0.0, (double) CavgR,
				   ah->Xdim, ah->Ydim);
        ovsunpx  = 1.0 / (float) Nsunpx;

	if (! (cifid = fopen (ciout, "w")) )
	    { fprintf (stderr, "%s can not be opened.\n", ciout); exit (1); }

	if (! (hifid = fopen (hiout, "w")) )
	    { fprintf (stderr, "%s can not be opened.\n", hiout); exit (1); }
        }

    mkfakedisk (fdimg, ah->Xdim, ah->Ydim, sunrad, inval, outval);
    fprintf (stdout, "Made fake disk of radius %.2f pixels\n\n", sunrad);
    

    /*** Main processing loop. ***/

    fseek (hfid, 0, 0);
    fseek (cfid, 0, 0);
    for (ii = 0; ii < Hims; ii++)
	{
        fscanf (hfid, "%s", Hf);
	fscanf (cfid, "%s", Cf);
        fprintf (stdout, "Files: %s %s\n", Hf, Cf);

	/* Continuum image. */

        readimg (Cf, cimg);
	rereadhdr (cimg, ah);
	zhdr (cimg);
	zero_shifts (cimg, ah);
        s2f (cimg, fcimg);		/* short -> float */

        if (calcBCC == 1)
	    { 
            chop_bot (cimg, imsz,   0);
            chop_top (cimg, imsz, 8192);
            annuli_pop (cimg, imsz, MEAN, AinfoC);
            BCC   = solve_LD_curve (AinfoC, Rmean, NANNULI, CavgR, &(LD->Cp));
            ovBCC = 1.0 / BCC;
            if (doprint == 1) fprintf (stderr, "File: %s BCC %.2f\n", Cf, BCC);
            }

        fvtimes (fcimg, ovBCC);
        if (calcIndex == 1)
	    {
	    LD->Cp.Index = SolarIndex (fcimg, Nsunpx, sunidx, ovsunpx);
	    fprintf (cifid, "%s %4.2f %6.3f %6.3f %10.2f %7.4f\n", 
		     Cf, LD->Cp.pop_frac, LD->Cp.a, LD->Cp.b, LD->Cp.c,
		     LD->Cp.Index);
	    }

	/* Helium image */

        readimg (Hf, himg);
	rereadhdr (himg, ah);
	zhdr (himg);
	zero_shifts (himg, ah);
        s2f (himg, fhimg);		/* short -> float */

        if (calcBCH == 1)
	    {
            chop_bot (himg, imsz,   0);
            chop_top (himg, imsz, 8192);
            annuli_pop (himg, imsz, INFL2, AinfoH);
            BCH   = solve_LD_curve (AinfoH, Rmean, NANNULI, HavgR, &(LD->Hp));
            ovBCH = 1.0 / BCH;
            if (doprint == 1) fprintf (stderr, "File: %s BCH %.2f\n", Hf, BCH);
            }

        fvtimes (fhimg, ovBCH);

        if (calcIndex == 1)
	    {
	    LD->Hp.Index = SolarIndex(fhimg, Nsunpx, sunidx, ovsunpx);
	    fprintf (hifid, "%s %4.2f %6.3f %6.3f %10.2f %7.4f\n", 
		     Hf, LD->Hp.pop_frac, LD->Hp.a, LD->Hp.b, LD->Hp.c, 
		     LD->Hp.Index);
	    }

        fminus (fhimg, fcimg);		/* Helium - Continuum */
        ftimes (fhimg, fdimg);		/* Helium * Fakedisk */

        if (do_rot )  do_frotate (fhimg, ah->Pangle); 
        if (mkflat)
	    {
	    HA = ( (double) ah->HA ) * 3.14159265358979323846 / 180.0 ;
	    air_factor = (float) ( 0.00002 / cos(HA) ) ;
	    if (doprint == 1) fprintf (stderr, "air_factor %.4g\n", air_factor);
	    MkFlat (fhimg, fdimg, air_factor);
	    }

        fvtimes (fhimg, sclfactor); /* Multiply up by 8192 to scale to short. */
        f2s (fhimg, himg);		/* float -> short */

	/*** Update header items. ***/

        ah->Type            = (ah->Type / 10) *10  + REDUCTION_m ;

        if (mkflat) ah->Type = (ah->Type / 10) *10  + REDUCTION_t ;
        if (do_rot) ah->Type = (ah->Type / 10) *10  + REDUCTION_h ;

        NameChip7 (afile, ah);

        replacehdr  (himg, ah);			/* Update image header. */
        insertLDinfo (himg, ah->Xdim, LD);
        writeimg (afile, himg);			/* Write image to disk. */
        }

    fclose (hfid);
    fclose (cfid);
    if (calcIndex == 1) { fclose (hifid); fclose (cifid); }
}

/* -------------------------------------------------------------------------- */

int
prsum ()
{
    static char sumry[] = "\
N_hsrMcsr [-R radius] [-BCC c_bri] [-BCH h_bri] [-f] [-r] [-H hsrlist] [-C csrlist] \n\
\n\
Example: N_hsrMcsr\n\
\n\
Example: N_hsrMcsr -BCC 3984. -BCH 4114. -H hsrlist -C csrlist \n\
\n\
Example: N_hsrMcsr -H hsrlist -C csrlist \n\
\n\
This code subtracts a continuum from a helium chip image using a\n\
The output files are: \"??????.dif\" \n\
If the \"-f\" flag is used, the output files are: \"??????.hsc\" \n\
\n\
Flags    Description\n\
-----    -----------\n\
 [-f]    Flatten the images, removing rotational trends.\n\
 [-r]    Images will be rotated to *heliocentric* coordinates.\n\
 [-H ]   The list of helium images (else the default \"hsrlist\" is used).\n\
 [-C ]   The list of continuum images (else the default \"csrlist\" is used).\n\
\n\
 [-R ]   The helium radius specified is used -- instead of an\n\
         average calculated from the entire list of helium images.\n\
 [-BCC ] The **user** specifies the brightness at the center of the\n\
         CONTINUUM images, useful for cloudy periods.\n\
 [-BCH ] The **user** specifies the brightness at the center of the\n\
         HELIUM    images, useful for cloudy periods.\n\
\n\
";

    fputs (sumry, stderr);
}

/* -------------------------------------------------------------------------- */

int
prhelp ()
{
    static char help[] = "\
N_hsrMcsr [-R radius] [-f] [-r] [-H hsrlist] [-C csrlist] \n\
\n\
Example: N_hsrMcsr\n\
\n\
Example: N_hsrMcsr -f -r -H hsclist -C csclist \n\
\n\
";
    fputs (help, stderr);
}

/* -------------------------------------------------------------------------- */

int
initLD (LD, BCC, BCH)

struct LDmodel_info *LD;
float BCC, BCH;
{
      LD->Hp.a = LD->Hp.b = LD->Cp.a = LD->Cp.b = 0.;
      LD->Hp.c = BCH ;
      LD->Cp.c = BCC ;
      LD->Hp.pop_frac = INFL2 ;
      LD->Cp.pop_frac = MEAN ;
      LD->Hp.Index    = LD->Cp.Index = 0.;
      LD->Nannuli     = NANNULI ;
}

/* -------------------------------------------------------------------------- */

float
SolarIndex (fimg, Nidx, pindx, scale)

    float *fimg; 
    int Nidx, *pindx; 
    float scale;
{
    int   ii, kk;
    int   *pim ;
    int   doprint=1;
    float solar_index;

    solar_index = 0.0 ;

    pim    = pindx ;
    for (ii = 0; ii < Nidx; ii++)
	{ 
	kk  = *pim;
	solar_index += *(fimg + kk); 
	pim++;
        }

    solar_index *= scale ;

    return (solar_index);
}

/* -------------------------------------------------------------------------- */

int
insertLDinfo (himg, xdim, LD)

short *himg;
int xdim;
struct LDmodel_info *LD;
{
    short *qq;
    float *fpi, *fps;
    int ii, Nitems;

    qq  = himg + xdim;
    fps = (float *)LD ;
    fpi = (float *)qq ; 

    Nitems = sizeof (struct LDmodel_info) / sizeof (float);

    for (ii = 0; ii < Nitems; ii++) *fpi++ = *fps++ ;

}

/* -------------------------------------------------------------------------- */
