/* NameChip7.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create file name for a CHIP image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * 17 January  2000   Alice Lecinski
 * 15 February 2000   ALS: modify for 7 filter position operation.
 * 08 Dec      2006   Alice 
		      Added a routine "Name_Chip"
		      for 2006_11_12_18_47_15.hsr filename style
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <ctype.h>

#include "acos.h"
#include "acos_header.h"


/*  Old, pre 7 wavelength naming style
static char wavelength[10][2] = {"x","h","c","m","w","a","b","r","k","s"};
static char Wavelength[10][2] = {"X","H","C","M","W","A","B","R","K","S"};
*/

static char wavelength[10][2] = {"x","h","d","t","c","s","b","r","k","o"};
static char      source[8][2] = {"x","s","l","o","d","b","c","f"};
static char   reduction[8][2] = {"x","r","c","l","d","m","t","h"};

static char Wavelength[10][2] = {"X","H","D","T","C","S","B","R","K","O"};
static char      Source[8][2] = {"X","S","L","O","D","B","C","F"};
static char   Reduction[8][2] = {"X","R","C","L","D","M","T","H"};


NameChip (filenm,a) char *filenm;  struct acos_header *a; {
    int doy,year;
    int ii,jj;

    /*
    if ( a->Version != 2 ) fprintf(stderr,"This is not CHIP data\n");
    */

    /* year = a->Year + 1900 ; */
    year = a->Year;
    doy = get_doy(a->Month,a->Day,year);

    sprintf(filenm,"%02dd%03d.%02d%02d%02d.%1s%1s%1s",
	(year >= 2000) ? year-2000 : year-1900,
	doy,
	a->Hour,
	a->Minute,
	a->Sec,
	wavelength[a->Type/100],
	source    [a->Type/10 - (a->Type/100)*10],
	reduction [a->Type - (a->Type/10)*10]);

}

Name_Chip (filenm,a) char *filenm;  struct acos_header *a; {
    int doy,year;
    int ii,jj;

    /*
    if ( a->Version != 2 ) fprintf(stderr,"This is not CHIP data\n");
    */

    year = a->Year;
    doy = get_doy(a->Month,a->Day,year);

    sprintf(filenm,"%4d_%02d_%02d_%02d_%02d_%02d.%1s%1s%1s",
	a->Year,
	a->Month,
	a->Day,
	a->Hour,
	a->Minute,
	a->Sec,
	wavelength[a->Type/100],
	source    [a->Type/10 - (a->Type/100)*10],
	reduction [a->Type - (a->Type/10)*10]);

}

int
get_doy(m, d, yr) int m, d, yr ;
{
    static int day_tab[2][13] = {
        {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
        {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
    };
    int ii , leap ;
 
    leap = yr%4 == 0 && yr%100 != 0 || yr%400 == 0 ;
    for ( ii=1 ; ii<m ; ii++ ) d += day_tab[leap][ii] ;
    return(d) ;
}
