/*
 *    ShowHeader
 *
 *    Read the current header and show it on the screen
 *    A user command to be entered at the keyboard to show
 *    the contents of the current data header.
 *    A good diagnostic tool.
 *
 *    David Elmore
 *
 *    Tue Jan 18 18:09:36 GMT 1994
    From kii: /local/usr/local/src/ACOS/Gui/Tasks
 */

#include    <stdio.h>
#include    <sys/mman.h>
#include    <fcntl.h>

#include    "logic.h"
#define        ACOS
#include    "acos.h"
#include    "acos_header.h"
#include    "image.h"
#define        START
#include    "mechanism.h"
#include    "mechanism_defs.h"
#include    "device.h"
#include    "lcvr.h"
#include    "analog.h"
#include    "mic.h"

extern        void    exit(),perror();

main (argc,argv)
int    argc;
char    *argv[];
{

int    i,buf,pos,dir,isum;
caddr_t    hdr_address[2],openheadermemory();
int    verbose = FALSE;
char    a,*p;

/* Open shared memory */

if ( (hdr_address[0] = openheadermemory()) < (caddr_t)0 )
    {
    printf ( "ShowHeader: memory open error\n" );
    exit (-1);
    }
hdr_address[1]=hdr_address[0]+NUM_COL*2;
Image_header[0] = (struct acos_header*) hdr_address[0];
Image_header[1] = (struct acos_header*) hdr_address[1];

if ( argc > 1 )
    {
    *++argv;
    printf ( "%s\n",*argv);
    p = *argv;
    if (*p++ == '-' )
        {
        if ( *p == 'v' )
            {
            verbose = TRUE;
            printf ("verbose is true\n" );
            }
        argc--;
        *++argv;
        }
    if ( argc > 1 )
        sscanf ( *argv,"%d",&buf );
    }
if ( (buf<0) || (buf>1) )
    buf = 0;

switch ( Image_header[buf]->Version )
    {
    case    IS_DPMON:
        printf ( "\n\ndPMon  " );
        break;
    case    IS_LOWK:
        printf ( "\n\nLowK   " );
        break;
    case    IS_CHIP:
        printf ( "\n\nCHIP   " );
        break;
    }
printf ( "%02d/%02d/%02d %02d:%02d:%02d    ",
        Image_header[buf]->Month,
        Image_header[buf]->Day,
        Image_header[buf]->Year,
        Image_header[buf]->Hour,
        Image_header[buf]->Minute,
        Image_header[buf]->Sec);
printf ( "Wavelength: %8.3f  ", Image_header[buf]->Wave );
printf ( "Type: %d\n", Image_header[buf]->Type );

printf ( "Image: %dx%d,   buffer %d  %d summed, %dmsec., %d gain factor\n",
        Image_header[buf]->Xdim,
        Image_header[buf]->Ydim,
        buf,
        Image_header[buf]->Nsum,
        Image_header[buf]->Exposure,
        Image_header[buf]->Gain );

printf ( "Occulting Disk Size %d arc-sec\n",Image_header[buf]->Odisk );
printf ( "Pangle: %7.3f  Bangle:    %7.3f  Langle:  %7.3f  Radius: %7.3f\n",
    Image_header[buf]->Pangle,Image_header[buf]->Bangle,
    Image_header[buf]->Langle,Image_header[buf]->Radius);
printf ( "HA:     %7.3f  Elevation: %7.3f  Azimuth: %7.3f  Amass:  %7.3f\n",
    Image_header[buf]->HA,Image_header[buf]->Elevation,
    Image_header[buf]->Azimuth,Image_header[buf]->Amass);
    
/*************
if ( (Image_header[buf]->Type > 0) && (Image_header[buf]->Type < NUM_TYPE) )
    printf ( "Observation Type %s\n",type_str[Image_header[buf]->Type] );
else
    printf ( "Observation Type Unknown\n" );
*****************/

if ( verbose )
    {
    isum = Image_header[buf]->Nsum;
    if ( (isum > 15) || (isum < 0 ) ) 
        isum = 1;
    for ( i=0 ; i<isum ; i++ )
        {
        printf ( "Xshift %8.3f   Yshift %8.3f   Diameter %8.3f\n",
            Image_header[buf]->Xshift[i],
            Image_header[buf]->Yshift[i],
            Image_header[buf]->Diam[i]);
        }


    for ( i=0 ; i<NUM_DEV ; i++ )
        {
        pos=Image_header[buf]->Pos[i];
        dir=Image_header[buf]->Drive[i];
        if ( (pos<0) || (pos>7))
            pos = 0;
        if ( (dir<0) || (dir>3))
            dir = 0;
        printf ( "%s    %s    %s\n",
        dev[i].name,dev[i].stat_name[pos],dev[i].drive_name[dir]);
        }

    printf ( "type <ret> to continue " );
    a = getchar();

    for ( i=0 ; i<NUM_A2D ; i++ )
        printf ("%s    %7.3f\n",analog[i].name,Image_header[buf]->Analog[i]);

    for ( i=0 ; i<NUM_LCVR ; i++ )
        printf ("%s    %7.3f\n",lcvr_name[i],Image_header[buf]->Lcvr[i]);

    for ( i=0 ; i<NUM_MIC ; i++ )
        printf ("%s    %7.1f\n",mic_name[i],Image_header[buf]->Mic[i]);
    }

printf ( "\n" );
}

