#include <stdio.h>
#include <math.h>
#include "annuli.h"



int
init_annuli(radpix)
    float radpix;
{
    int ii;
    double ovN,r0,ri,ro,rarea;
    double  pi=3.14159265358979323846 ;

    ovN = 1./(NANNULI);  ovN -= 0.0001;
    r0  = sqrt ( ovN );

    ii=0;
    ri=0.;
    ro=r0*(double)radpix;
    while (ro <= 1.*radpix) {
	Rinner[ii]=ri;
	Router[ii]=ro;
	Rmean [ii]=sqrt( (ri*ri + ro*ro)*0.5 ) ;
	rarea = pi * ro*ro  - pi * ri*ri ;
	ri = ro ;   ro = sqrt((rarea + pi*ri*ri) / pi) ;
	ii++;
    }

}
int
get_annuli_idx(indx,inrad,outrad,xdim,ydim)
    int *indx;
    double inrad,outrad;
    int   xdim,ydim;
{
    int i, j , *q;
    int Npts;
    int doprint=0;
    double  xcen , ycen ;
    double  x , y , xdis , ydis , xdissq , ydissq , radius ;
    double sqrt() ;
    double ovsunrad;

    Npts=0;

    xcen = ((double)xdim-1.)*0.5;
    ycen = ((double)ydim-1.)*0.5;
    q = indx;
    x = y = 0 ;
    for (i=0;i<xdim*ydim; i++ ) {
	
	x = (double) (i%xdim) ;     y = (double) (i/xdim) ;

	xdis = xcen - x ;     xdissq = xdis*xdis  ;
	ydis = ycen - y ;     ydissq = ydis*ydis  ;

	radius = sqrt ( xdissq + ydissq) ;
	if (radius<outrad && radius>=inrad) { *q++ = i  ; Npts++;  }
    }
    /*
    if(doprint==1) {
	fprintf(stderr,"\t\tget_annuli_idx, %6.1f %6.1f %5d %5d, %8d\n",
		       inrad,outrad,xdim,ydim,Npts);
    }
    */
    return(Npts);
}
int
setup_annuli(radpix,xdim,ydim)
    float radpix;
    int   xdim, ydim;
{
    int ii;
    int doprint=0;


    /*
    if(doprint==1) fprintf(stderr,"\tSet up %d %d\n",xdim,ydim );
    */

    a_idx =  a_indicies ;

    init_annuli(radpix);
    for(ii=0;ii<NANNULI;ii++){
	/*
	if(doprint==2) {
	    fprintf(stderr,
	    "\tSet up %d \t Ri Ro %6.1f %6.1f \t Radius %6.1f\n",
	    ii,Rinner[ii],Router[ii],radpix);
	}
	*/
	Na_pix[ii] = get_annuli_idx(a_idx[ii],Rinner[ii],Router[ii],xdim,ydim);
	/*
	if(doprint==1) 
	    fprintf(stderr,"\tSet_up %d \t first index %d\n",ii,*a_idx[ii]);
	*/
    }
}

int annuli_pop(img,imsz,toppop,ainfo) 
    short *img; 
    int imsz; 
    float toppop;
    double *ainfo;
{
    short *qq,*pp;
    int   ii,kk,nn,Npts;
    int   imin,imax,hist_sz;
    int   *pim , *hist_i;
    int   req_pop, pop;
    int   doprint=2;
    double *point;

    hist_i =  HIST;
    point  =  ainfo;

    for(nn=0;nn<NANNULI;nn++){
	Npts = Na_pix[nn];
	/*
	if(doprint==1) fprintf(stderr,"annuli_pop %d Npts %d\n",nn,Npts);
	*/

	a_idx  = a_indicies ;
	pim    = a_idx[nn];
	qq     = a_img;  
	/*
	if(doprint==1) 
	    fprintf(stderr,"annuli_pop %d \t first index %d\n",nn,*pim);
	*/
	for(ii=0;ii<Npts;ii++){ 
	    kk  = *pim;
	    *qq = *(img + kk); 
	    qq++ ;
	    pim++;
	}
	imin=array_min(a_img,Npts);
	imax=array_max(a_img,Npts);

	hist_sz = 1+(imax-imin);
	/*
	if(doprint==1) 
	    fprintf(stderr,"annuli_pop %d \t\thist_sz %d\n",nn,hist_sz);
	*/

	pim = hist_i; for(ii=0;ii<RANGE;ii++) *pim++ = 0;

	histogram(a_img,hist_i,Npts,imin);

	req_pop = (int)(toppop * (float) Npts);
	pop=0; 
	ii  = (imax-imin);
	pim = hist_i + ii;
	while(pop < req_pop) {
	    pop += *pim;
	    pim--;
	    ii--;
	}
	ii++;  ii += imin;  /* ii is the numerical value where the population
			       meets the desired criteria. */


        *point++ = (double) ii;

    }
    /*
    if(doprint==2){
	point  =  ainfo;
	for(nn=0;nn<NANNULI;nn++){
	    fprintf(stderr," %6.2f %6.2f",*point++,Rmean[nn]);
	}
	fprintf(stderr,"\n");
    }
    */

}
