/**************** findshiftL ***********************/

/*   Code to find x and y shift to "center" any "disk" type image.     */
/*   This is iterative, recalculating the centers until the difference */
/*       from the last calculation is less than 0.5 pixels.            */
/*                                 Alice Lecinski,  January 1996       */

/*   Pass this code --> img     -- your short image.
                       numx        the x-dimension of your "img"
                       numy        the y-dimension of your "img"
                       xshi        This will be found by findshiftL.
                                       It is the x-shift from the center
                                       in fractional pixels.
                       yshi        This will be found by findshiftL.
                                       It is the y-shift from the center
                                       in fractional pixels.
                       diam        An approximate first guess at the "diam"
                                       *must* be passed to findshiftL.
                                       The true diameter is returned.
*/
/******************************************************************************/

#include <stdio.h>
#define ILIMIT      (10)  /* Iterations before failure */
#define NSPOTS      (10)
#define LIMIT       (2.5)  /* pixel separation between previous and present
                              center computation. */
#define MNSOB    (  100.) /* The sobel minimum must be less than this,
                             or it isn't a real edge. was -1500. */

float   ledge[NSPOTS], redge[NSPOTS], *le, *re;
float     bot[NSPOTS],   top[NSPOTS], *be, *te;



int findshiftL(img,numx,numy,xshi,yshi,diam)
    short *img;
    float *xshi,*yshi, *diam;
    int numx,numy;
{

    long ii,jj,kk,iter, strt;
    float xcen,ycen, sepx,sepy, XCEN,YCEN, goodx,goody, goodt,goodb;
    float lastx,lasty;
    float *ximg,*yimg,*sximg,*syimg;
    register float *qq, *pp, *ss;
    register short *iq;
    float sxmn,sxmx,symx,symn;
    float sobel[11]={ -6.,-2.,0.,2.,4.,4.,4.,2.,0.,-2.,-6.};
    int   hs=5, whs=11;
    int notgoodenough;
    int iycn , ixcn;
    int xx , yy ;
    float RE,LE,TE,BE;
    float xshift,yshift,t_diam;
    float cavg;
    double yedge;
    double xdiff, ydiff, xydiff, fabs();

    notgoodenough=1;
    iter=0;
    xx = 350; yy = 20;

    ximg  = (float *) malloc ( numx*     sizeof ( float )) ;
    sximg = (float *) malloc ( numx*     sizeof ( float )) ;
    yimg  = (float *) malloc ( numy*     sizeof ( float )) ;
    syimg = (float *) malloc ( numy*     sizeof ( float )) ;

    XCEN = xcen = lastx = (numx-1.)*0.5;
    YCEN = ycen = lasty = (numy-1.)*0.5;



    /* Find a rough xcen */
        ixcn = (int)xcen;  iycn = (int)ycen;      cavg=0.;
        pp=  ximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        strt=iycn;
        qq=  ximg; 
        iq=img+strt*numx; for(ii=0;ii<numx;ii++) *qq++ = *iq++;
         
        qq=ximg+ixcn-50;
        for(ii=0;ii<50*2;ii++) cavg += *qq++;
        cavg /= 101.;  cavg = cavg + cavg*0.40;

        pp=ximg+ixcn; ii=ixcn;
        while((*pp<cavg)&&(ii>xx)) { ii--; pp--;}
        LE=ii;

        pp=ximg+ixcn; ii=ixcn; 
        while((*pp<cavg)&&(ii<numx-xx)){ ii++; pp++;}
        RE=ii;
        xcen = (RE-LE)*0.5 +LE;
        ixcn = (int)xcen;
        /* fprintf(stderr,"LE RE xcen %f %f %f\n",LE,RE,xcen); */

    /* Find a rough ycen */
        ixcn = (int)xcen;  iycn = (int)ycen;      cavg=0.;
        pp=  yimg; for(ii=0;ii<numy;ii++) *pp++=0.;
        strt = ixcn  - 50;
        qq=  yimg; 
        iq=img+strt; for(ii=0;ii<numy;ii++){ *qq = *iq; qq++; iq+=numx; }

        qq=yimg+iycn-50;
        for(ii=0;ii<50*2;ii++) cavg += *qq++;
        cavg /= 101.;  cavg = cavg + cavg*0.40;

        pp=yimg+iycn; ii=iycn;
        while((*pp<cavg)&&(ii>yy)) { ii--; pp--;}
        BE=ii;

        pp=yimg+iycn; ii=iycn; 
        while((*pp<cavg)&&(ii<numy-yy)){ ii++; pp++;}
        TE=ii;
        ycen = (TE-BE)*0.5 +BE;
        iycn = (int)ycen;

    while(notgoodenough) {

        pp=  ximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp=  yimg; for(ii=0;ii<numy;ii++) *pp++=0.;


        /* Find the left and right edges. */

        strt = (int)(ycen-NSPOTS*0.5);  /* Get to the "middle" of the image. */
        re=redge; le=ledge;
        for(kk=0;kk<NSPOTS;kk++){

            qq=  ximg; 
            iq=img+(strt+kk)*numx; for(ii=0;ii<numx;ii++) *qq++ = *iq++;

            cavg=0.;
            ixcn = (int)xcen;
            qq=ximg+ixcn-200;    for(ii=0;ii<200*2;ii++) cavg += *qq++;
            cavg /= 401.;  cavg = cavg + cavg*0.40;

            pp=ximg+ixcn; ii=ixcn;
            while((*pp<cavg)&&(ii>xx)) { ii--; pp--;}
            *le++=(float)ii;

            pp=ximg+ixcn; ii=ixcn;
            while((*pp<cavg)&&(ii<numx-xx)){ ii++; pp++;}
            *re++=(float)ii;

        }


        /* Find the top and bottom edges. */

        strt = (int)(xcen-NSPOTS*0.5)  - 50;
        be=bot; te=top;
        for(kk=0;kk<NSPOTS;kk++){

            qq=  yimg; 
            iq=img+strt+kk; for(ii=0;ii<numy;ii++){ *qq = *iq; qq++; iq+=numx; }

            cavg=0.;
            iycn = (int)ycen;
            qq=yimg+iycn-200;    for(ii=0;ii<200*2;ii++) cavg += *qq++;
            cavg /= 401.;  cavg = cavg + cavg*0.40;


            pp=yimg+iycn; ii=iycn;
            while((*pp<cavg)&&(ii>yy)) { ii--; pp--;}
            *be++=(float)ii;

            pp=yimg+iycn; ii=(int)iycn;
            while((*pp<cavg)&&(ii<numy-yy)){ ii++; pp++;}
            *te++=(float)ii;
        }

        /*  Check the separations between the top and bottom
            and the right and left edges.  The separations should be close 
            to the known diameter of the sun for the instrument.  If not
            the edge was found incorrectly.  In particular, this happens
            for the bottom and top edges when the image of the sun is 
            moved around in the fov to find the flat field. 
        */
        RE=LE=BE=TE=0.; 
        re=redge; le=ledge; be=bot; te=top;
        /****
        fprintf(stderr,"\n");
        */
        goodx=goody=0.;
        for (ii=0;ii<NSPOTS;ii++){
            /****
            fprintf(stderr,"%10.2f %10.2f \t %10.2f %10.2f\n",*le,*re,*be,*te);
            */
            sepx= *re - *le;
            if(sepx < (*diam)+0.06*(*diam) && sepx > (*diam)-0.06*(*diam)) {
                LE += *le ; RE += *re ;
                /****
                fprintf(stderr,"Good LR %10.2f %10.2f \t",*le,*re);
                */
                goodx++;
            }
            /****
            else fprintf(stderr,"Bad LR %10.2f %10.2f \t",*le,*re);
            */
            le++;re++;
            sepy= *te - *be;
            if(sepy < (*diam)+0.06*(*diam) && sepy > (*diam)-0.06*(*diam)) {
                BE += *be ; TE += *te ;
                /****
                fprintf(stderr,"Good BT %10.2f %10.2f \n",*be,*te);
                */
                goody++;
            }
            /****
            else fprintf(stderr,"Bad BT %10.2f %10.2f \n",*be,*te);
            */
            be++;te++;
        }
        if(goodx != 0. ) { LE /= goodx; RE /= goodx; xcen = (RE-LE)*0.5 +LE; }
        else             {                           xcen = lastx + 2.*LIMIT; }
        if(goody != 0. ) { BE /= goody; TE /= goody; ycen = (TE-BE)*0.5 +BE; }
        else             {                           ycen = lasty + 2.*LIMIT; }
                         

        t_diam= (RE-LE)*1.01; /* Can't use top bot because of pylon. */
        *xshi = xshift = XCEN-xcen;
        *yshi = yshift = YCEN-ycen;

        ++iter;
        /********
        fprintf(stderr,"Iteration: %d\n",iter);
        fprintf(stderr,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
        fprintf(stderr,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
        fprintf(stderr,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
        *********/
      

        /* Check to see how close the centers are to the previous iteration.
           Note, the while loop will happen at least twice...
        */
        xdiff = lastx-xcen;  xdiff = fabs(xdiff);
        ydiff = lasty-ycen;  ydiff = fabs(ydiff);
        xydiff = xdiff + ydiff;
        if (xydiff < LIMIT) notgoodenough=0;
        if (iter   > ILIMIT) return(0);

        lastx=xcen; lasty=ycen;

    }

    *diam = t_diam;

    /****
    fprintf(stderr,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
    fprintf(stderr,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
    fprintf(stderr,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
    fprintf(stderr,"\tDiameter: %10.2f Radius %10.2f\n",*diam,*diam*0.5);
    ****/

    free( (char *) ximg );
    free( (char *)sximg );
    free( (char *) yimg );
    free( (char *)syimg );

    return(1);
}





int findshiftL_sobel(img,numx,numy,xshi,yshi,diam)
    short *img;
    float *xshi,*yshi, *diam;
    int numx,numy;
{

    long ii,jj,kk,iter, strt;
    float xcen,ycen, sepx,sepy, XCEN,YCEN, goodx,goody, goodt,goodb;
    float lastx,lasty;
    float *ximg,*yimg,*sximg,*syimg;
    register float *qq, *pp, *ss;
    register short *iq;
    float sxmn,sxmx,symx,symn;
    float sobel[11]={ -6.,-2.,0.,2.,4.,4.,4.,2.,0.,-2.,-6.};
    int   hs=5, whs=11;
    int notgoodenough;
    int iycn , ixcn;
    int xx , yy ;
    float RE,LE,TE,BE;
    float xshift,yshift,t_diam;
    float cavg;
    double yedge;
    double xdiff, ydiff, xydiff, fabs();

    notgoodenough=1;
    iter=0;
    xx = 350; yy = 20.;

    ximg  = (float *) malloc ( numx*     sizeof ( float )) ;
    sximg = (float *) malloc ( numx*     sizeof ( float )) ;
    yimg  = (float *) malloc ( numy*     sizeof ( float )) ;
    syimg = (float *) malloc ( numy*     sizeof ( float )) ;

    XCEN = xcen = lastx = (numx-1.)*0.5;
    YCEN = ycen = lasty = (numy-1.)*0.5;


    iycn = (int)ycen;
    ixcn = (int)xcen;

    while(notgoodenough) {

        pp= sximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp=  ximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp= syimg; for(ii=0;ii<numy;ii++) *pp++=0.;
        pp=  yimg; for(ii=0;ii<numy;ii++) *pp++=0.;


        /* Find the left and right edges. */

        strt = (int)(ycen-NSPOTS*0.5);  /* Get to the "middle" of the image. */
        re=redge; le=ledge;
        for(kk=0;kk<NSPOTS;kk++){

            /**** Zero the X-sobel area, and grab a X row from the image. */

            pp= sximg;             for(ii=0;ii<numx;ii++) *pp++=0.;
            qq=  ximg; 
            iq=img+(strt+kk)*numx; for(ii=0;ii<numx;ii++) *qq++ = *iq++;


            /**** Apply the sobel to the X-line. */
            pp=sximg+hs;
            for(ii=hs;ii<numx-hs;ii++){
                ss=sobel;
                qq=ximg+ii-hs;
                for(jj=0;jj<whs;jj++) { *pp += (*ss) * (*qq) ; ss++; qq++; }
                pp++;
            }

            /**** Find the minimum (the edge) on the left           */
            /**** Gotta go backwards.                               */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=sximg+ixcn+whs;
            sxmn=0.;
            for(ii=0;ii<XCEN-xx;ii++){ sxmn= (sxmn < *pp) ? sxmn : *pp; pp--; }
            ii=ixcn+whs; pp=sximg+ixcn+whs;   
                                    while( *pp-- > sxmn ) ii--;
                                    while( *pp-- <    0 ) ii--;

            *le++=(float)(ii) + (0.-*pp)/((float)(*(pp+1)-*(pp)));

            /****
            fprintf(stderr,"left %10.2f   sxmn=%10.2f\t",*(le-1), sxmn);
            */
 
            /**** Find the minimum (the edge) on the right          */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=sximg+ixcn-whs;
            sxmn=0.;
            for(ii=0;ii<XCEN-xx;ii++){ sxmn= (sxmn < *pp) ? sxmn : *pp; pp++; }
            ii=ixcn-whs; pp=sximg+ixcn-whs;     while( *pp++ > sxmn ) ii++;
                                                while( *pp++ <    0 ) ii++;  
            *re++=(float)(ii-1) + (0.-*(pp-1))/((float)(*pp-*(pp-1)));

            /****
            fprintf(stderr,"right %10.2f  sxmn=%10.2f\n",*(re-1),sxmn);
            */
        }





        /* Find the top and bottom edges. */

        strt = (int)(xcen-NSPOTS*0.5)  - 50;
        be=bot; te=top;
        for(kk=0;kk<NSPOTS;kk++){

            /**** Zero the Y-sobel area, and grab a Y row from the image. */
            pp= syimg;      for(ii=0;ii<numy;ii++)  *pp++=0.;
            qq=  yimg; 
            iq=img+strt+kk; for(ii=0;ii<numy;ii++){ *qq = *iq; qq++; iq+=numx; }
            pp=syimg+hs;

            /**** Apply the sobel to the Y-line. */
            for(ii=hs;ii<numy-hs;ii++){ 
                ss=sobel;
                qq=yimg+ii-hs;
                for(jj=0;jj<whs;jj++) *pp += *ss++ * *qq++ ;
                pp++;
            }

            /**** Find the minimum (the edge) on the bottom         */
            /**** Gotta go backwards.                               */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=syimg+whs+iycn;
            symn= 0.;  /*If it isn't less than this, it isn't a real edge*/
            for(ii=0;ii<YCEN;ii++){ symn=(symn < *pp) ? symn : *pp; pp--; }
            if (symn < MNSOB)  {
                ii=whs+iycn; pp=syimg+whs+iycn;   
                                        while( *pp-- > symn ) ii--;
                                        while( *pp-- <    0 ) ii--;
                *be++= (float)(ii) + (0.-*pp)/((float)(*(pp+1)-*(pp)));
            }
            else *be++ = numy;

            /****
            fprintf(stderr,"bottom %10.2f  symn=%10.2f\t",*(be-1),symn);
            ****/
 
            /**** Find the minimum (the edge) on the top            */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=syimg+iycn-whs;
            symn= 0.;
            for(ii=0;ii<YCEN;ii++){ symn=(symn < *pp) ? symn : *pp; pp++; }
            if (symn < MNSOB)  {
                ii=iycn-whs; pp=syimg+iycn-whs;  
                                                     while( *pp++ > symn ) ii++;
                                                     while( *pp++ <    0 ) ii++;
                *te++=(float)(ii-1) + (0.-*(pp-1))/((float)(*pp-*(pp-1)));
            }
            else *te++ = 0.;

            /****
            fprintf(stderr,"top %10.2f   symn=%10.2f\n",*(te-1),symn);
            ****/
        }

        /*  Check the separations between the top and bottom
            and the right and left edges.  The separations should be close 
            to the known diameter of the sun for the instrument.  If not
            the edge was found incorrectly.  In particular, this happens
            for the bottom and top edges when the image of the sun is 
            moved around in the fov to find the flat field. 
        */
        RE=LE=BE=TE=0.; 
        re=redge; le=ledge; be=bot; te=top;
        /****
        fprintf(stderr,"\n");
        */
        goodx=goody=0.;
        for (ii=0;ii<NSPOTS;ii++){
            /****
            fprintf(stderr,"%10.2f %10.2f \t %10.2f %10.2f\n",*le,*re,*be,*te);
            */
            sepx= *re - *le;
            if(sepx < (*diam)+0.06*(*diam) && sepx > (*diam)-0.06*(*diam)) {
                LE += *le ; RE += *re ;
                /****
                fprintf(stderr,"Good LR %10.2f %10.2f \t",*le,*re);
                */
                goodx++;
            }
            /****
            else fprintf(stderr,"Bad LR %10.2f %10.2f \t",*le,*re);
            */
            le++;re++;
            sepy= *te - *be;
            if(sepy < (*diam)+0.06*(*diam) && sepy > (*diam)-0.06*(*diam)) {
                BE += *be ; TE += *te ;
                /****
                fprintf(stderr,"Good BT %10.2f %10.2f \n",*be,*te);
                */
                goody++;
            }
            /****
            else fprintf(stderr,"Bad BT %10.2f %10.2f \n",*be,*te);
            */
            be++;te++;
        }
        if(goodx != 0. ) { LE /= goodx; RE /= goodx; xcen = (RE-LE)*0.5 +LE; }
        else             {                           xcen = lastx + 2.*LIMIT; }
        if(goody != 0. ) { BE /= goody; TE /= goody; ycen = (TE-BE)*0.5 +BE; }
        else             {                           ycen = lasty + 2.*LIMIT; }
                         

        t_diam= (RE-LE)*1.01; /* Can't use top bot because of pylon. */
        *xshi = xshift = XCEN-xcen;
        *yshi = yshift = YCEN-ycen;

        ++iter;
        /********
        fprintf(stderr,"Iteration: %d\n",iter);
        fprintf(stderr,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
        fprintf(stderr,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
        fprintf(stderr,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
        *********/
      

        /* Check to see how close the centers are to the previous iteration.
           Note, the while loop will happen at least twice...
        */
        xdiff = lastx-xcen;  xdiff = fabs(xdiff);
        ydiff = lasty-ycen;  ydiff = fabs(ydiff);
        xydiff = xdiff + ydiff;
        if (xydiff < LIMIT) notgoodenough=0;
        if (iter   > ILIMIT) return(0);

        lastx=xcen; lasty=ycen;

    }

    *diam = t_diam;

    /****
    fprintf(stderr,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
    fprintf(stderr,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
    fprintf(stderr,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
    fprintf(stderr,"\tDiameter: %10.2f Radius %10.2f\n",*diam,*diam*0.5);
    ****/

    free( (char *) ximg );
    free( (char *)sximg );
    free( (char *) yimg );
    free( (char *)syimg );

    return(1);
}
