/**************** findshift ***********************/

/*   Code to find x and y shift to "center" any "disk" type image.     */
/*   This is iterative, recalculating the centers until the difference */
/*       from the last calculation is less than 0.5 pixels.            */

/*   Pass this code --> img     -- your short image.
                       numx        the x-dimension of your "img"
                       numy        the y-dimension of your "img"
                       xshi        the x shift in fractional pixels 
                                   found by this code
                       yshi        the y shift in fractional pixels 
                                   found by this code
                       diam        the true diameter found by this code.

    Updated 10/21/97 to offset from the top   and bottom
                                   and  right and left
    to get past "edge" problems.  (xoff and yoff)
 
    Also, narrowed the diameter fractional acceptance limit to
    0.10 from 0.36.    0.1 is still around 65 pixels, pretty big. (dlim)
    --Alice

*/
/******************************************************************************/

#include <stdio.h>
#define BUG
#define ILIMIT      (10)  /* Iterations before failure */
#define NSPOTS      (10)
#define LIMIT       (0.5)  /* pixel separation between previous and present
                              center computation. */
#define MNSOB    ( -750. ) /* The sobel minimum must be less than this,
                             or it isn't a real edge. was -1500. */
                           /* -750 finds no limbs */

float   ledge[NSPOTS], redge[NSPOTS], *le, *re;
float     bot[NSPOTS],   top[NSPOTS], *be, *te;



void findshiftPICS(img,numx,numy,xshi,yshi,diam)
    short *img;
    float *xshi,*yshi, *diam;
    int numx,numy;
{

    float xcen,ycen, sepx,sepy, XCEN,YCEN, goodx,goody, goodt,goodb;
    float lastx,lasty;
    float *ximg,*yimg,*sximg,*syimg;
    float sxmn,sxmx,symx,symn;
    float dlim;  /* Fraction of how big the diameter can be off. */
    /* float sobel[11]={ -6.,-2.,0.,2.,4.,4.,4.,2.,0.,-2.,-6.}; */
    float sobel[11]={ -3.,-2.,0.,2.,2.,2.,2.,2.,0.,-2.,-3.};
    int   hs=5, whs=11;
    int   yoff , xoff;
    int notgoodenough;
    float RE,LE,TE,BE;
    float xshift,yshift,t_diam;
    double yedge;
    double xdiff, ydiff, xydiff, fabs();
    register float 	*qq, *pp, *ss;
    register short 	*iq;
    register	int	 ii,jj,kk,iter,strt;

    notgoodenough=1;
    iter=0;
    xoff = 100;
    yoff = 50;
    dlim = 0.36;
    dlim = 0.10;

    ximg  = (float *) malloc ( numx*     sizeof ( float )) ;
    sximg = (float *) malloc ( numx*     sizeof ( float )) ;
    yimg  = (float *) malloc ( numy*     sizeof ( float )) ;
    syimg = (float *) malloc ( numy*     sizeof ( float )) ;

    XCEN = xcen = lastx = (numx-1.)*0.5;
    YCEN = ycen = lasty = (numy-1.)*0.5;

    while(notgoodenough) {

        pp= sximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp=  ximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp= syimg; for(ii=0;ii<numy;ii++) *pp++=0.;
        pp=  yimg; for(ii=0;ii<numy;ii++) *pp++=0.;

        /* Find the left and right edges. */


        strt = (int)(ycen-NSPOTS*0.5);  /* Get to the "middle" of the image. */
        re=redge; le=ledge;
        for(kk=0;kk<NSPOTS;kk++){

            /**** Zero the x-sobel area, and grab a x row from the image. */

            pp= sximg;             for(ii=0;ii<numx;ii++) *pp++=0.;
            qq=  ximg; 
            iq=img+(strt+kk)*numx; for(ii=0;ii<numx;ii++) *qq++ = *iq++;


            /**** Apply the sobel. */
            pp=sximg+hs;
            for(ii=hs;ii<numx-hs;ii++){
                ss=sobel;
                qq=ximg+ii-hs;
                for(jj=0;jj<whs;jj++) { *pp += (*ss) * (*qq) ; ss++; qq++; }
                pp++;
            }

            /**** Find the minimum (the edge) on the right          */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=sximg+whs+xoff;
            sxmn=0.;
            for(ii=xoff;ii<XCEN;ii++){ sxmn= (sxmn < *pp) ? sxmn : *pp; pp++; }

            ii=whs+xoff; pp=sximg+whs+xoff;
		while( *pp++ > sxmn ) ii++;
                while( *pp++ <    0 ) ii++;
            *le++=(float)(ii-1) + (0.-*(pp-1))/((float)(*pp-*(pp-1)));

#ifdef BUG
            fprintf(stdout,"left %10.2f   sxmn=%10.2f\t",*(le-1), sxmn);
#endif
 
            /**** Find the minimum (the edge) on the right          */
            /**** Gotta go backwards.                               */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=sximg+numx-1-whs-xoff;
            sxmn=0.;
            for(ii=xoff;ii<XCEN;ii++){ sxmn= (sxmn < *pp) ? sxmn : *pp; pp--; }
            ii=numx-1-whs-xoff; pp=sximg+numx-1-whs-xoff; 
		while( *pp-- > sxmn ) ii--;
                while( *pp-- <    0 ) ii--;

            *re++=(float)(ii) + (0.-*pp)/((float)(*(pp+1)-*(pp)));

#ifdef BUG
            fprintf(stdout,"right %10.2f  sxmn=%10.2f\n",*(re-1),sxmn);
#endif
        }





        /* Find the top and bottom edges. */

        strt = (int)(xcen-NSPOTS*0.5) ;
        be=bot; te=top;
        for(kk=0;kk<NSPOTS;kk++){

            pp= syimg;      for(ii=0;ii<numy;ii++)  *pp++=0.;
            qq=  yimg; 
            iq=img+strt+kk; for(ii=0;ii<numy;ii++){ *qq = *iq; qq++; iq+=numx; }
            pp=syimg+hs;

            for(ii=hs;ii<numy-hs;ii++){ 
                ss=sobel;
                qq=yimg+ii-hs;
                for(jj=0;jj<whs;jj++) *pp += *ss++ * *qq++ ;
                pp++;
            }

            pp=syimg+whs+yoff;
            symn= MNSOB;  /*If it isn't less than this, it isn't a real edge*/
            for(ii=yoff;ii<YCEN;ii++){ symn=(symn < *pp) ? symn : *pp; pp++; }
            if (symn < MNSOB)  {
                ii=whs+yoff; pp=syimg+whs+yoff;
		    while( *pp++ > symn ) ii++;
                    while( *pp++ <    0 ) ii++;
                *be++=(float)(ii-1) + (0.-*(pp-1))/((float)(*pp-*(pp-1)));
            }
            else *be++ = numy;

#ifdef BUG
            fprintf(stdout,"bottom %10.2f  symn=%10.2f\t",*(be-1),symn);
#endif
 
            pp=syimg+numy-1-whs-yoff;
            symn= MNSOB;
            for(ii=0;ii<YCEN;ii++){ symn=(symn < *pp) ? symn : *pp; pp--; }
            if (symn < MNSOB)  {
                ii=numy-1-whs-yoff; pp=syimg+numy-1-whs-yoff;  
		    while( *pp-- > symn ) ii--;
                    while( *pp-- <    0 ) ii--;
                *te++= (float)(ii) + (0.-*pp)/((float)(*(pp+1)-*(pp)));
            }
            else *te++ = 0.;

#ifdef BUG
            fprintf(stdout,"top %10.2f   symn=%10.2f\n",*(te-1),symn);
#endif
        }

        /*  Check the separations between the top and bottom
            and the right and left edges.  The separations should be close 
            to the known diameter of the sun for the instrument.  If not
            the edge was found incorrectly.  In particular, this happens
            for the bottom and top edges when the image of the sun is 
            moved around in the fov to find the flat field. 
        */
        RE=LE=BE=TE=0.; 
        re=redge; le=ledge; be=bot; te=top;

#ifdef BUG
        fprintf(stdout,"\n");
#endif

        goodx=goody=0.;
        for (ii=0;ii<NSPOTS;ii++){
#ifdef BUG
            fprintf(stdout,"%10.2f %10.2f \t %10.2f %10.2f",*le,*re,*be,*te);
#endif
            sepx= *re - *le;
            if(sepx < (*diam)+dlim*(*diam) && sepx > (*diam)-dlim*(*diam)) {
                LE += *le ; RE += *re ;
                goodx++;
#ifdef BUG
		fprintf(stdout," goodx ");
#endif
            }
            sepy= *te - *be;
            if(sepy < (*diam)+dlim*(*diam) && sepy > (*diam)-dlim*(*diam)) {
                BE += *be ; TE += *te ;
                goody++;
#ifdef BUG
		fprintf(stdout," goody ");
#endif
            }
		le++ ; re++ ; be++ ; te++ ;
#ifdef BUG
		fprintf(stdout,"\n");
#endif
        }
        if(goodx != 0. ) {  LE /= goodx; RE /= goodx; }
	else             {  LE = 0.;     RE =  0.;    }
	if(goody != 0. ) {  BE /= goody; TE /= goody; }
	else             {  BE = 0.;     TE = 0.;     }
        xcen  = (RE-LE)*0.5 +LE;
        ycen  = (TE-BE)*0.5 +BE;
                         
                        /* If the y separations were hideous, go back and */
        if(goody==0) {  /* find a good top **or** bottom edge. */
            BE=TE=0.; 
            goodt=goodb=0.;
            be=bot; te=top;
            for (ii=0;ii<NSPOTS;ii++){
                if((*be>whs ) && (*be<YCEN)    ) { BE += *be++ ; goodb++; }
                if((*te>YCEN) && (*te<numy-whs)) { TE += *te++ ; goodt++; }
            }
            if      (goodb>0.) { 
                BE /= goodb; yedge=(double)BE; BE=0.; 
                goodb=0.;
                be=bot;
                for (ii=0;ii<NSPOTS;ii++){
                    ydiff=yedge- (double)*be;
                    if( fabs(ydiff) < 0.1*numy ) { BE += *be++ ; goodb++; }
                }
                BE /= goodb;  TE=BE+(*diam);  ycen=BE+0.5*(*diam);
            }
            else if (goodt>0.) {
                TE /= goodt; yedge=(double)TE; TE=0.; 
                goodt=0.;
                te=top;
                for (ii=0;ii<NSPOTS;ii++){
                    ydiff=yedge- (double)*te;
                    if( fabs(ydiff) < 0.1*numy ) { TE += *te++ ; goodt++; }
                }
                TE /= goodt;  BE=TE-(*diam);  ycen=TE-0.5*(*diam);
            }
            else {
#ifdef BUF
                fprintf(stdout,"Neither a good top or bottom was found.\n");
#endif
                TE=YCEN+(*diam)*0.5 + 2.*LIMIT*(float)iter;
                BE=YCEN-(*diam)*0.5 + 2.*LIMIT*(float)iter;
                ycen  = (TE-BE)*0.5 +BE;
            }
        }

        t_diam= ((RE-LE) + (TE-BE))*0.5;
        *xshi = xshift = XCEN-xcen;
        *yshi = yshift = YCEN-ycen;

#ifdef BUG
        fprintf(stdout,"Iteration: %d\n",iter);
        fprintf(stdout,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
        fprintf(stdout,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
        fprintf(stdout,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
#endif
      

        /* Check to see how close the centers are to the previous iteration.
           Note, the while loop will happen at least twice...
        */
        xdiff = lastx-xcen;  xdiff = fabs(xdiff);
        ydiff = lasty-ycen;  ydiff = fabs(ydiff);
        xydiff = xdiff + ydiff;
        if (xydiff < LIMIT) notgoodenough=0;

        if (iter   > ILIMIT) 
			{
			*xshi = 0.;
			*yshi = 0.;
			return;
			}

        lastx=xcen; lasty=ycen;

	iter++;
    }

    *diam = t_diam;

#ifdef BUG
    fprintf(stdout,"findshift: iterations: %d\n",iter);
    fprintf(stdout,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
    fprintf(stdout,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
    fprintf(stdout,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
    fprintf(stdout,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
#endif

    free( (char *) ximg );
    free( (char *)sximg );
    free( (char *) yimg );
    free( (char *)syimg );
}
