/*
                norm_accumulate (filenm,img)

                Each img is normalized to an intensity
                level of NORM for the center BSZxBSZ pixels.

                This routine reads a list of files in "filenm"
                It reads in each of these files and
                accumulates them into "img" .
*/


#include                <stdio.h>
#include                "dim.h"


int
norm_accumulate (filenm,img)   char *filenm;    short *img;
{
    int   ii,jj,kk,got,doprint=0;
    float *accum,ovnum;
    float rndr=0.5005;    /* Value to round up to nearest integer. */
    FILE  *accumlist, *fid;
    char  imgfile[120];

    register float *ap;
    register short *ip;

    float  tot,ovtot;
    double ravg,sigma,ravgsq,sigmasq,npts, val, hlimit, llimit;
    double sqrt();
    int    xfirst,yfirst,xcen,ycen;

    if(doprint) fprintf(stderr,"norm_accumulate:\n");

    xcen = (ncol-1)*0.5;
    ycen = (nrow-1)*0.5;
    xfirst = xcen-FBSZ*0.5;
    yfirst = ycen-FBSZ*0.5;

    accum = (float *) malloc ((size_t)(sizeof(float)*imsz));

    /* Zero out the accum area. */
    ap=accum;  for(ii=0;ii<imsz;ii++) *ap++ = 0.;

    if ( ! ( accumlist = fopen ( filenm,"r" ) ) ) {
        fprintf(stderr,"norm_accumulate: file %s cannot be opened\n",filenm);
        exit(-99);
    }

    /* Loop through all the files in accumlist */
    /*      open the file and sum into a float */

    for(kk=0; ;kk++) {

        fscanf(accumlist,"%s",imgfile);    if(feof(accumlist)!=0) break;
	
	if(doprint) fprintf(stderr,"\t%2d: %s ",kk+1,imgfile);
        if (! (fid = fopen (imgfile,"r")) ) {
            fprintf(stderr,"norm_accumulate: %s does not exist.\n",imgfile);
            exit(1);
        }

        if((got=fread(img,1,im_nbytes,fid))!=im_nbytes) {
            fprintf(stderr,
                    "norm_accumulate: read error in %s expected %d got %d\n",
                    imgfile,im_nbytes,got );
            exit(2);
        }
        fclose(fid);

	/* First compute the mean and standard deviation. */
        ravg   = 0.;
        ravgsq = 0.;
	npts = FBSZ*FBSZ;
        for (jj=yfirst ;jj<yfirst+BSZ ;jj++ ) {
            ip = img + xfirst +jj*ncol;
            for (ii=0;ii<BSZ;ii++)  { ravg   += (float)*ip; 
				      ravgsq += (float)*ip * (float)*ip ;
				      ip++; }
        }
	sigmasq = ( npts*ravgsq - ravg*ravg ) / ( npts*(npts-1));
	sigma   = 2.*sqrt(sigmasq) ;
	ravg   /= (FBSZ*FBSZ) ; 
        if(doprint) fprintf ( stderr,"xm s %7.2f %7.2f  ",ravg,sigma );
	hlimit  = ravg + sigma;
	llimit  = ravg - sigma;
	/* Now go back through the area, and toss those points outside
	   2*sigma. This is to exclude non-functioning diodes. */
	tot=0.;
	npts=0.;
        for (jj=yfirst ;jj<yfirst+BSZ ;jj++ ) {
            ip = img + xfirst +jj*ncol;
            for (ii=0;ii<BSZ;ii++)  { 
		val   = (double)*ip; 
		if ((val < hlimit) && (val > llimit)) {
		    tot += val;
		    npts++;
		}
		ip++; }
        }

        ovtot = NORM*(npts)/tot;
        if(doprint) fprintf ( stderr,"Total: %7.2f  Tossed %.0f\n",
			       ovtot,FBSZ*FBSZ-npts );

        ip = img;
        ap = accum;
        for(ii=0;ii<imsz;ii++) {   *ap += ((float)(*ip))*ovtot;  ip++; ap++; }

    }
    fclose(accumlist);

    ovnum = 1./(float)kk;

    /* Divide by the number of images read */
    ap = accum;
    ip = img;
    for(ii=0;ii<imsz;ii++)      {  *ip = (short)(rndr + (*ap)*ovnum);
                                      ip++;
                                      ap++;
    }
    free((char *)accum);

}



int
norm_accumulateSIGMALIMIT (filenm,img,limit)
    char *filenm;
    short *img;
    float limit;
{
    int   ii,jj,kk,got,doprint=0;
    float *accum,ovnum;
    float rndr=0.5005;    /* Value to round up to nearest integer. */
    FILE  *accumlist, *fid;
    char  imgfile[120];

    register float *ap;
    register short *ip;

    float  tot,ovtot;
    double ravg,sigma,ravgsq,sigmasq,npts, val, hlimit, llimit;
    double sqrt();
    int    xfirst,yfirst,xcen,ycen;

    if(doprint) fprintf(stderr,"norm_accumulate:\n");

    xcen = (ncol-1)*0.5;
    ycen = (nrow-1)*0.5;
    xfirst = xcen-FBSZ*0.5;
    yfirst = ycen-FBSZ*0.5;

    accum = (float *) malloc ((size_t)(sizeof(float)*imsz));

    /* Zero out the accum area. */
    ap=accum;  for(ii=0;ii<imsz;ii++) *ap++ = 0.;

    if ( ! ( accumlist = fopen ( filenm,"r" ) ) ) {
        fprintf(stderr,"norm_accumulate: file %s cannot be opened\n",filenm);
        exit(-99);
    }

    /* Loop through all the files in accumlist */
    /*      open the file and sum into a float */

    for(kk=0; ;kk++) {

        fscanf(accumlist,"%s",imgfile);    if(feof(accumlist)!=0) break;
	
	if(doprint) fprintf(stderr,"\t%2d: %s ",kk+1,imgfile);
        if (! (fid = fopen (imgfile,"r")) ) {
            fprintf(stderr,"norm_accumulate: %s does not exist.\n",imgfile);
            exit(1);
        }

        if((got=fread(img,1,im_nbytes,fid))!=im_nbytes) {
            fprintf(stderr,
                    "norm_accumulate: read error in %s expected %d got %d\n",
                    imgfile,im_nbytes,got );
            exit(2);
        }
        fclose(fid);

	/* First compute the mean and standard deviation. */
        ravg   = 0.;
        ravgsq = 0.;
	npts = FBSZ*FBSZ;
        for (jj=yfirst ;jj<yfirst+BSZ ;jj++ ) {
            ip = img + xfirst +jj*ncol;
            for (ii=0;ii<BSZ;ii++)  { ravg   += (float)*ip; 
				      ravgsq += (float)*ip * (float)*ip ;
				      ip++; }
        }
	sigmasq = ( npts*ravgsq - ravg*ravg ) / ( npts*(npts-1));
	sigma   = limit*sqrt(sigmasq) ;
	ravg   /= (FBSZ*FBSZ) ; 
        if(doprint) fprintf ( stderr,"xm s %7.2f %7.2f  ",ravg,sigma );
	hlimit  = ravg + sigma;
	llimit  = ravg - sigma;
	/* Now go back through the area, and toss those points outside
	   limit*sigma. This is to exclude non-functioning diodes. */
	tot=0.;
	npts=0.;
        for (jj=yfirst ;jj<yfirst+BSZ ;jj++ ) {
            ip = img + xfirst +jj*ncol;
            for (ii=0;ii<BSZ;ii++)  { 
		val   = (double)*ip; 
		if ((val < hlimit) && (val > llimit)) {
		    tot += val;
		    npts++;
		}
		ip++; }
        }

        ovtot = NORM*(npts)/tot;
        if(doprint) fprintf ( stderr,"Total: %7.2f  Tossed %.0f\n",
			       ovtot,FBSZ*FBSZ-npts );

        ip = img;
        ap = accum;
        for(ii=0;ii<imsz;ii++) {   *ap += ((float)(*ip))*ovtot;  ip++; ap++; }

    }
    fclose(accumlist);

    ovnum = 1./(float)kk;

    /* Divide by the number of images read */
    ap = accum;
    ip = img;
    for(ii=0;ii<imsz;ii++)      {  *ip = (short)(rndr + (*ap)*ovnum);
                                      ip++;
                                      ap++;
    }
    free((char *)accum);

}
