/*
                readimg  (filenm,img)
                writeimg (filenm,img)

                Reads  an array "img" from the file "filenm"
		or
                Writes an array "img"  to  the file "filenm"

		"img" is assumed to be a short array of size "imsz"
*/


#include                <stdio.h>
#include                "dim.h"
#include                "params.h"

int
printit(astring) char *astring;
{
    fprintf(stderr,"\t%s\t",astring);
}
int
readimg(filenm,img) char *filenm; short *img; 
{
    int  ii,doprint=0;
    char *p;
    FILE *fid;

    is_fits=0;
    is_dpmn=0;

    if(doprint) fprintf(stderr,"\t\tReading %s",filenm);

    /* Check for a fits file... */
    p= filenm+strlen(filenm) -3;
    if( *p=='f' && *(p+1)=='t' && *(p+2)=='s') is_fits=1;

    if(doprint){
	if(is_fits==1) fprintf(stderr,"   Fits file\n");
	else           fprintf(stderr,"   \n");
    }

    /* Check for a dpmn fits file... */
    if(is_fits==1) {
	p= filenm+strlen(filenm)-26;
	if( *p=='d' && *(p+1)=='p' && *(p+2)=='m' && *(p+3)=='n' ) is_dpmn=1;
    }

    if(doprint){
	if(is_fits==1) fprintf(stderr,"   Fits file\n");
	else           fprintf(stderr,"   \n");
    }

    if ( ( fid = fopen(filenm,"r") ) == NULL  ) {
	fprintf(stderr,"Error openning %s for reading.\n",filenm);
	exit(-90);
    }

    if(is_fits==1) fseek(fid,2880*2,0L);

    if( ( ii = fread (img,sizeof(short),imsz,fid)) != imsz ) {
	fprintf(stderr,"Only read %d shorts from %s\nExpected %d",
		        ii,filenm,imsz);
	exit(-92);
    }
    if(is_fits==1 && is_dpmn==0) flipTB(img+ncol,ncol,nrow-1);
    fclose(fid);
}

int
writeimg(filenm,img) char *filenm; short *img; 
{
    int  ii,doprint=0;
    FILE *fid;

    /* Check for a fits file... */
    /* Don't activate this until later when the fits header update
       is working right!!
    p= filenm+strlen(filenm) -3;
    if( *p=='f' && *(p+1)=='t' && *(p+2)=='s') is_fits=1;
    */

    if(is_fits==1 && is_dpmn==0) flipTB(img+ncol,ncol,nrow-1);

    if(doprint) fprintf(stderr,"\t\tWriting %s\n",filenm);
    if ( ( fid = fopen(filenm,"w") ) < (FILE*)0 ) {
	fprintf(stderr,"Error openning %s for writing. Error %d\n",filenm,ii);
	exit(-90);
    }
    if( ( ii = fwrite (img,sizeof(short),imsz,fid)) != imsz ) {
	fprintf(stderr,"Only wrote %d shorts from %s\nExpected %d",
			ii,filenm,imsz);
	exit(-92);
    }
    fclose(fid);
}

void
writefvalue(filenm,value)
    char *filenm;
    float value;
{
    int  ii,doprint=0;
    char *w;
    FILE *fid;

    if(doprint) fprintf(stderr,"\t\tWriting %s\n",filenm);
    if ( ( fid = fopen(filenm,"w") ) < (FILE*)0 ) {
	fprintf(stderr,"Error openning %s for writing. Error %d\n",filenm,ii);
	exit(-90);
    }

	fprintf(fid,"%10.2f\n",value);
    fclose(fid);
}



#include "shift.h"
void
writeshft_diam(filenm,list,shft,num,len)
    char *filenm, *list;
    SHFT shft;
    int  num,len;
{
    int  ii,doprint=0;
    char *w;
    FILE *fid;

    if(doprint) fprintf(stderr,"\t\tWriting %s\n",filenm);
    if ( ( fid = fopen(filenm,"w") ) < (FILE*)0 ) {
	fprintf(stderr,"Error openning %s for writing. Error %d\n",filenm,ii);
	exit(-90);
    }

    w=list;
    for(ii=0;ii<num;ii++){
	fprintf(fid,"%s: %10.2f %10.2f %10.2f\n",w,shft->X,shft->Y,shft->Dia);
	w+=len;
	shft++;
    }
    fclose(fid);
}

int
flipTB(img,xxdim,yydim)
    short *img;
    int   xxdim,yydim;
{
    extern char *malloc () ;
    short *rimg, *qq, *rq;
    int ii, jj, imsz;

    imsz=xxdim*yydim;
 
    rimg = (short *)malloc(imsz*sizeof(short));
 
    /* Flip top to bottom */
    qq=img; rq=rimg+imsz - xxdim;
    for(jj=0;jj<yydim;jj++) {
        for(ii=0;ii<xxdim;ii++) *rq++ = *qq++ ;
        rq -= 2*xxdim;
    }
 
    /* Copy it back. */
    qq=img; rq=rimg;        for(jj=0;jj<imsz;jj++)  *qq++ = *rq++ ;
 
 
    free((char *)rimg);
 
}
int
fflipTB(img,xxdim,yydim)
    float *img;
    int   xxdim,yydim;
{
    extern char *malloc () ;
    float *rimg, *qq, *rq;
    int ii, jj, imsz;

    imsz=xxdim*yydim;
 
    rimg = (float *)malloc(imsz*sizeof(float));
 
    /* Flip top to bottom */
    qq=img; rq=rimg+imsz - xxdim;
    for(jj=0;jj<yydim;jj++) {
        for(ii=0;ii<xxdim;ii++) *rq++ = *qq++ ;
        rq -= 2*xxdim;
    }
 
    /* Copy it back. */
    qq=img; rq=rimg;        for(jj=0;jj<imsz;jj++)  *qq++ = *rq++ ;
 
 
    free((char *)rimg);
 
}
