/* 
 *        ReceiveDisplayCommand
 *
 *        Receive a command on the Display Command Queue.
 *        int        wait_flag        If != 0 then only pick up a message
 *                                if there and return.
 *                                If == 0 wait for a message then return it.
 *
 *
 *        A routine to pick up command from the Display command queue
 *        Used by display to know what to do.
 *        Sent from the Program task, manually from Command selection
 *        or from keyboard via SendDisplayCommand instruction.
 *        Two arguments, the shift to apply to the data and the buffer
 *        to display.  Normally 1 and 0.  See Display for more.
 *
 *        David Elmore
 *
 *        Wed Feb  2 16:21:15 GMT 1994
 */

#define                MESSAGE
#include        "message.h"

extern                void        exit();

main (argc,argv)
int        argc;
char        *argv[];
{
int        code,shift,buf;
int        msgflg = 0;
char        *str,*receivemessage();


if ( argc > 1 )
        {
        *argv++;
        code = strtoul (*argv,NULL,0);
        if ( code != 0 )
                msgflg = IPC_NOWAIT;
        else
                msgflg = 0;
        }
else
        msgflg = 0;

/* Receive the message */
/* If argument was 0 then we wait here until message is received */
/* otherwise if there is a message we grab it, and if not go on */

str = receivemessage ( DISPLAY_COMMAND,msgflg );
code = str;

if ( code > 0 )
        {
        sscanf ( str,"%d %d",&shift,&buf );
/*        fprintf ( stdout,"ReceiveDisplayCommand %d %d\n",shift,buf );*/
        }
else
        code = 99;

exit(code);
}

/* 
 *        char*        receivemessage ( key,msgflag )
 *
 *        int        key                Message Key, see message.h
 *        int        msgflag                IPC_NOWAIT or 0 for wait
 *        returns null terminated pointer to string of up to 25 characters
 *
 *        Receive message on specified queue.
 *
 *        David Elmore
 *
 *        Wed Feb  2 16:21:15 GMT 1994
 */

extern                void        perror();

char *
receivemessage        (key,msgflag)
key_t        key;
int        msgflag;
{
static        char        string[MSG_NBYTES];
int        messagesize = MSG_NBYTES;
int        i,rtrn;

/* Is the queue we want not yet defined? */

if ( (msgqid[key] = msgget (key,0)) < 0 )
        {
        if ( (msgqid[key] = msgget ( key,IPC_CREAT|IPC_EXCL|0666 )) < 0 )
                {
                perror("sendmessage open failure\n");
                return (-1);
                }
        }

/* Allocate room for a message with all its baggage */

if ( msgp <= 0 )
        {
        msgp = (struct msgbuf *)malloc((unsigned)(sizeof(struct msgbuf) -
                sizeof msgp->mtext + MSG_NBYTES ));
        if ( msgp == NULL )
                {
        fprintf(stderr,"sendmessage: could not malloc %d bytes\n",MSG_NBYTES);
                return (-1);
                }
        }

/* Receive the message */

rtrn = msgrcv(msgqid[key],msgp,MSG_NBYTES, 0, msgflag);
if ( rtrn == -1 )
        return ( -1 );

/* copy message to another buffer */
for ( i=0 ; i<MSG_NBYTES ; i++ )
        string[i] = msgp->mtext[i];

return ( string );
}
