/*
                limb diam del_diam >limb_file

                diam is a file containing an ascii floating point diameter
                or is a constant with a leading + sign.

                del_diam is a floating point constant to be added
                to the value of diam.

                This routine generates a limb darkened disk
                of diameter specified in the call.

                The limb profile uses the formula given in Allen
                and the constants are set for 1 micron.

*/


#include                <stdio.h>
#include                <stdlib.h>
#include                <errno.h>
#include                <math.h>
#include                "dim.h"

main(argc,argv)
char                **argv;
int                argc;
{
    char  *p;

    /* Determined emperically 10/16/95 DFE: approx .8 micron from Allen */

    float  a = 0.97;   /* Bigger moves fall off to limb */
    float  b = 0.20;   /* Bigger increases magnitude of fall off */

    /* .80 and .30 dark in the middle */
    /* .60 and .38 is flat with roll off toto */
    /* .95 and .20 is flat and goes closer to limb */
    /* 1.18 and 0.10 is flat and goes very close to limb, maybe a little
       roll off too soon*/
    /* 1.08 and 0.15 looks good */
    /* 1.35 and 0.00 is bright in the middle */
    /* 1.06 and 0.15 work with limb reduced by 5 pixels */
    /* might want to bring down center a little */

    float          c;
    FILE           *diamfile;
    int            isafile,i,j,code,write();
    short          *img;
    float          deldiam,fi,fj,pow,diam,val,x,y,xcen,ycen,rad,rvec;
    register short *ip;

    int            nrow,ncol,record;

    sscanf ( getenv ( "XDIM" ),"%d",&ncol );
    sscanf ( getenv ( "YDIM" ),"%d",&nrow );
    record = 2*ncol;
    if ( ncol == 0 ) {
        ncol = NCOL;
        nrow = NROW;
        record = RECORD;
    }
    img = (short*)malloc ( (size_t)(2*ncol*nrow) );
    /*fprintf ( stderr,"%d %d\n",ncol,nrow );*/


    c = (1. - a - b )/.306853;

    deldiam = 0.;
    isafile = 0;
    if ( argc > 1 ) {
        p = *++argv;
        if ( ( *p != '+' ) && ( *p != '-' ) ) {
	    isafile = 1;
	    if ( ( diamfile = fopen ( p,"r" ) ) < (FILE *)0 ) {
		fprintf ( stderr," open error %d %s\n",diamfile,p );
		goto usage;
	    }
	    if ( ( code = fscanf ( diamfile,"%f",&diam ) ) < 0 ) {
		if ( code == -1 ) {
		    fprintf ( stderr,"limb: read error %d %s\n",code,p );
		    goto usage;
		}
	    }
	}
        else {
	    sscanf ( p,"%f",&diam );
	}
        if ( argc > 2 ) {
	    p = *++argv;
	    sscanf ( p,"%f",&deldiam );
	}
    }
    else {
	usage:
	    fprintf (stderr,"usage: limb arg  (arg is a file or +/-const.)\n" );
	    exit ( 1 );
    }

    rad = (diam+deldiam)/2.;

    /*
    xcen = (ncol-1.)/2.;
    ycen = (nrow-1.)/2.;
    */
    xcen = (ncol)/2.;
    ycen = (nrow)/2.;

    ip = img;

    for ( i=0; i<nrow ; i++ ) {
        fi = i;
        y = (fi-ycen)/rad;
        for ( j=0 ; j<ncol ; j++ ) {
	    fj = j;
	    x = (fj-xcen)/rad;
	    rvec = sqrt(x*x + y*y);
	    if ( rvec >= 1. ) {
		*ip++ = 8192.*0.5;
	    }
	    else {
		rvec = asin(rvec);
		rvec = cos(rvec);
		val = a;
		val = val + b*rvec;
		val = val + c*(1-rvec*log(1.+1./rvec));
		val = val * 8192.;
		*ip++ = (short)val;
	    }
        }
    }

    if ( ( code = write ( 1,img,record*nrow ) ) != record*nrow ) {
	fprintf ( stderr," write error %d \n",code );
	exit ( 1 );
    }
}
