/*
                do_shift
                
                A filter to shift an image.
                do_shift shiftfile < in_image > out_image

                shiftfile is a file containing the 
                        xshift, yshift, and diam values (floats)

                It calls Alice's subroutine doshift() to
                shift an image the specified amount
*/


#include                <stdio.h>
#include                <stdlib.h>
#include                <errno.h>
#include                "dim.h"

main(argc,argv)
char                **argv;
int                argc;
{
    FILE           *file;
    void           doshift();
    char           *p;
    short          *img;
    int            i,j,code,read(),write();
    float          xshi,yshi,diam;
    float          *fimg;
    register float *fp;
    register short *ip;
    
    int            nrow,ncol,record,im_nbytes,imsize;

    sscanf ( getenv ( "XDIM" ),"%d",&ncol );
    sscanf ( getenv ( "YDIM" ),"%d",&nrow );
    record = 2*ncol;
    if(ncol==0) {ncol=NCOL;nrow=NROW;record=RECORD;}
    imsize    = nrow*ncol;
    im_nbytes = sizeof(short)*imsize;

    img  = (short*)malloc ( (size_t)(2*imsize) );
    fimg = (float*)malloc ( (size_t)(4*imsize) );

    /*fprintf ( stderr,"%d %d\n",ncol,nrow );*/
    
    if ( argc == 2 ) {
	p = *++argv;
	if ( ( file = fopen ( p,"r" ) ) < (FILE*)0 ) {
	    fprintf ( stderr," open error %d %s\n",file,p );
	    goto usage;
	}
	fscanf ( file,"%f %f %f",&xshi,&yshi,&diam );
	/*  fprintf ( stderr,"%7.2f %7.2f %7.2f\n",xshi,yshi,diam );*/
    }
    else
        goto usage;
    
    if ( ( code = read( 0,img,im_nbytes ) ) != im_nbytes ) {
	fprintf ( stderr,"do_shift: read error %d\n",code );
	exit ( 1 );
    }
    
    doshift(img,fimg,ncol,nrow,xshi,yshi);
    
    ip = img;
    fp = fimg;
    for ( j=0 ; j<imsize ; j++ )     *ip++ = (short)(*fp++ +0.5);
    
    if ( ( code = write ( 1,img,im_nbytes) ) != im_nbytes ) {
	fprintf ( stderr," write error %d \n",code );
	exit ( 1 );
    }
    
    exit ( 0 );
    
    usage:
	fprintf (stderr, "do_shift shiftfile < in_image > out_image\n" );
	fprintf (stderr, "\'shiftfile\' contains xshift, yshift, and diam\n");
	exit (-1);
    
}

/********************** doshift ***********************/

/*   Code to "center" any "disk" type image.  This code applies the x and y
     shifts found by routine "findshift()". */

/*   Pass this code --> img     -- your short image.
                       fimg     -- a spot in memory to accumulate your
                                   recentered image. same size as your "img"
                                   above, but a float.
                       numx        the x-dimension of your "img"
                       numy        the y-dimension of your "img"
                       xshi        the x shift in fractional pixels 
                                       found by findshift()
                       yshi        the y shift in fractional pixels 
                                       found by findshift()
*/

#include <stdio.h>

void doshift(img,fimg,numx,numy,xshi,yshi)
    short *img;
    float *fimg , xshi,yshi;
    int numx,numy;
{

    long ii,jj,kk,strt;
    long imsz;
    register float *ff, *tt;
    float xx,xxm1,yy,yym1;
    float *timg;
    register float *t1,*t2,*t3,*t4;
    register float *qq, *pp, *ss;
    register short *iq;
    int shift, off;
    float xshift,yshift;

    /* Copy the short image to a float image. */
    imsz=numx*numy;      timg = (float *) malloc ( imsz*sizeof ( float )) ;

    tt= timg; iq=img; for(ii=0;ii<imsz;ii++) *tt++ = *iq++;


    /* Get the fractional and integer portions of the x and y shift. */
    xshift = xshi ;
    yshift = yshi ;
    if(xshift < 0.0) {
        xx = 1. + (xshift - (int)xshift); xxm1=1.-xx;
        xshift -= 1.0;
    }
    else {
        xx = xshift-(int)xshift; xxm1=1.-xx;
    }
    if(yshift < 0.0) {
        yy = 1. + (yshift - (int)yshift); yym1=1.-yy;
        yshift -= 1.0;
    }
    else {
        yy = yshift-(int)yshift; yym1=1.-yy;
    }
    shift = (int)yshift*numx+(int)xshift;  /* This is how much to shift
					      the entire array in whole
					      integer dimensions, looking
					      at the array as if it was 1-d. */

    /*   Position to the integer portion of the shift. */
    if(shift>=0) { off=shift;        ff=fimg+off; tt=timg;     }
    else         { off = shift * -1; ff=fimg;     tt=timg+off; }


    tt += numx; ff += numx;
    t1=tt-1;
    t2=tt;
    t3=tt-1-numx;
    t4=tt  -numx;

    strt = imsz-off-numx;                  /* Apply the fractional portion */
    for(ii=0;ii<strt;ii++)                 /* of the shift. */
        *ff++ +=  *t1++ * xx  *yym1 + 
                  *t2++ * xxm1*yym1 + 
                  *t3++ * xx  *yy   + 
                  *t4++ * xxm1*yy           ;

    free( (char *) timg );
}
