/*
                norm_accumulate filelist > accumulated_file

                This program reads a list of files
                It reads in each of these files and
                accumulates them.
                Each image is normalized to an intensity
                level of 8192 for the center 128x128 pixels.

                The accumulated file is sent to stdout.
*/


#include                <stdio.h>
#include                <stdlib.h>
#include                <errno.h>
#include                <fcntl.h>
#include                "dim.h"

main(argc,argv)
char                **argv;
int                argc;
{
    FILE   *accumlist;
    char   filename[80],*p;
    short  *img,stemp;
    int    fdimg,i,j,k,code,read(),write();
    int    xfirst,yfirst,xcen,ycen;
    double *accum,ovnum,tot,dtemp;

    register double *ap;
    register short  *ip;
    
    
    int            nrow,ncol,record,im_nbytes,imsize;

    sscanf ( getenv ( "XDIM" ),"%d",&ncol );
    sscanf ( getenv ( "YDIM" ),"%d",&nrow );
    record = 2*ncol;
    if(ncol==0) {ncol=NCOL;nrow=NROW;record=RECORD;}
    imsize    = nrow*ncol;
    im_nbytes = sizeof(short)*imsize;

    /*fprintf ( stderr,"%d %d\n",ncol,nrow );*/

    img   = (short *)malloc ( (size_t)(im_nbytes) );
    accum = (double*)malloc ( (size_t)(sizeof(double)*imsize) );
    
    if ( argc == 2 ) {
        p = *++argv;
        if ( ( accumlist = fopen ( p,"r" ) ) < (FILE*)0 ) {
            fprintf ( stderr," open error %d %s\n",accumlist,p );
            goto usage;
        }
    }
    else
        goto usage;
    
    xcen = (ncol-1)/2;
    ycen = (nrow -1)/2;
    xfirst = xcen-64;
    yfirst = ycen-64;

    ap = accum;  for ( j=0 ; j<imsize ; j++ ) *ap++ = 0.;
    
    /* Loop through all the file names in accumlist */
    /* open the file named there and sum into a float */
    
    code = 0;
    for ( k=0 ; feof(accumlist) == 0 ; k++ ) {

        fscanf ( accumlist,"%s\n",filename );
        fdimg = open ( filename,O_RDONLY );
        fprintf ( stderr,"norm_accumulate: %s ",filename );
        if((code=read(fdimg,img,im_nbytes))!=im_nbytes){
            fprintf(stderr,
		    "norm_accumulate: read error in %s expected %d got %d\n",
                    filename,im_nbytes,code );
            exit(1);
        }
        
        tot = 0.;
        for ( j=yfirst ; j<yfirst+128 ; j++ ) {
            ip = img + xfirst + j*ncol;
            for (i=0;i<128;i++)  { tot += (double)*ip; ip++; }
        }
        tot = 8192.*(128.*128.)/tot;
    
        ip = img;
        ap = accum;
        for (j=0; j<imsize ; j++ )  {  *ap   += ((double)*ip)*tot;
					ip++;
					ap++;
	}
        fprintf ( stderr,"Total: %7.2f \n",tot );
    }
    fclose ( accumlist );

    ovnum  = 1./(double)k;
    
    /* Normalize by the number of images read */
    ap = accum;
    ip = img;
    for (j=0; j<imsize ; j++ )      {  *ip = (short)(0.5 + (*ap)*ovnum);
					ip++;
					ap++;
    }
    
    write ( 1,img,im_nbytes);
    
    exit ( 0 );
    
    usage:
        fprintf (stderr, "accumulate filelist >accumulated_image\n" );
        exit (-1);
    
}
